@@ -14,8 +14,8 @@
 # limitations under the License.
 
 import json
-from xml.sax import saxutils
 
+from swift.common.middleware import listing_formats
 from swift.common.swob import HTTPOk, HTTPNoContent
 from swift.common.utils import Timestamp
 from swift.common.storage_policy import POLICIES
@@ -78,43 +78,27 @@ def account_listing_response(account, req, response_content_type, broker=None,
 
     account_list = broker.list_containers_iter(limit, marker, end_marker,
                                                prefix, delimiter, reverse)
-    if response_content_type == 'application/json':
-        data = []
-        for (name, object_count, bytes_used, put_timestamp, is_subdir) \
-                in account_list:
-            if is_subdir:
-                data.append({'subdir': name})
-            else:
-                data.append(
-                    {'name': name, 'count': object_count,
-                     'bytes': bytes_used,
-                     'last_modified': Timestamp(put_timestamp).isoformat})
-        account_list = json.dumps(data)
-    elif response_content_type.endswith('/xml'):
-        output_list = ['<?xml version="1.0" encoding="UTF-8"?>',
-                       '<account name=%s>' % saxutils.quoteattr(account)]
-        for (name, object_count, bytes_used, put_timestamp, is_subdir) \
-                in account_list:
-            if is_subdir:
-                output_list.append(
-                    '<subdir name=%s />' % saxutils.quoteattr(name))
-            else:
-                item = '<container><name>%s</name><count>%s</count>' \
-                       '<bytes>%s</bytes><last_modified>%s</last_modified>' \
-                       '</container>' % \
-                       (saxutils.escape(name), object_count,
-                        bytes_used, Timestamp(put_timestamp).isoformat)
-                output_list.append(item)
-        output_list.append('</account>')
-        account_list = '\n'.join(output_list)
+    data = []
+    for (name, object_count, bytes_used, put_timestamp, is_subdir) \
+            in account_list:
+        if is_subdir:
+            data.append({'subdir': name.decode('utf8')})
+        else:
+            data.append(
+                {'name': name.decode('utf8'), 'count': object_count,
+                 'bytes': bytes_used,
+                 'last_modified': Timestamp(put_timestamp).isoformat})
+    if response_content_type.endswith('/xml'):
+        account_list = listing_formats.account_to_xml(data, account)
+        ret = HTTPOk(body=account_list, request=req, headers=resp_headers)
+    elif response_content_type.endswith('/json'):
+        account_list = json.dumps(data).encode('ascii')
+        ret = HTTPOk(body=account_list, request=req, headers=resp_headers)
+    elif data:
+        account_list = listing_formats.listing_to_text(data)
+        ret = HTTPOk(body=account_list, request=req, headers=resp_headers)
     else:
-        if not account_list:
-            resp = HTTPNoContent(request=req, headers=resp_headers)
-            resp.content_type = response_content_type
-            resp.charset = 'utf-8'
-            return resp
-        account_list = '\n'.join(r[0] for r in account_list) + '\n'
-    ret = HTTPOk(body=account_list, request=req, headers=resp_headers)
+        ret = HTTPNoContent(request=req, headers=resp_headers)
     ret.content_type = response_content_type
     ret.charset = 'utf-8'
     return ret
