@@ -24,7 +24,7 @@ import six
 from six.moves.urllib import parse as urlparse
 
 from swift import gettext_ as _
-from swift.common.exceptions import EncryptionException
+from swift.common.exceptions import EncryptionException, UnknownSecretIdError
 from swift.common.swob import HTTPInternalServerError
 from swift.common.utils import get_logger
 from swift.common.wsgi import WSGIContext
@@ -40,7 +40,7 @@ class Crypto(object):
     cipher = 'AES_CTR_256'
     # AES will accept several key sizes - we are using 256 bits i.e. 32 bytes
     key_length = 32
-    iv_length = algorithms.AES.block_size / 8
+    iv_length = algorithms.AES.block_size // 8
 
     def __init__(self, conf=None):
         self.logger = get_logger(conf, log_route="crypto")
@@ -78,9 +78,9 @@ class Crypto(object):
             # The CTR mode offset is incremented for every AES block and taken
             # modulo 2^128.
             offset_blocks, offset_in_block = divmod(offset, self.iv_length)
-            ivl = long(binascii.hexlify(iv), 16) + offset_blocks
+            ivl = int(binascii.hexlify(iv), 16) + offset_blocks
             ivl %= 1 << algorithms.AES.block_size
-            iv = str(bytearray.fromhex(format(
+            iv = bytes(bytearray.fromhex(format(
                 ivl, '0%dx' % (2 * self.iv_length))))
         else:
             offset_in_block = 0
@@ -89,7 +89,7 @@ class Crypto(object):
                         backend=self.backend)
         dec = engine.decryptor()
         # Adjust decryption boundary within current AES block
-        dec.update('*' * offset_in_block)
+        dec.update(b'*' * offset_in_block)
         return dec
 
     def create_iv(self):
@@ -155,7 +155,7 @@ class CryptoWSGIContext(WSGIContext):
         self.logger = logger
         self.server_type = server_type
 
-    def get_keys(self, env, required=None):
+    def get_keys(self, env, required=None, key_id=None):
         # Get the key(s) from the keymaster
         required = required if required is not None else [self.server_type]
         try:
@@ -165,11 +165,14 @@ class CryptoWSGIContext(WSGIContext):
             raise HTTPInternalServerError(
                 "Unable to retrieve encryption keys.")
 
+        err = None
         try:
-            keys = fetch_crypto_keys()
+            keys = fetch_crypto_keys(key_id=key_id)
+        except UnknownSecretIdError as err:
+            self.logger.error('get_keys(): unknown key id: %s', err)
+            raise
         except Exception as err:  # noqa
-            self.logger.exception(_(
-                'ERROR get_keys(): from callback: %s') % err)
+            self.logger.exception('get_keys(): from callback: %s', err)
             raise HTTPInternalServerError(
                 "Unable to retrieve encryption keys.")
 
@@ -191,6 +194,17 @@ class CryptoWSGIContext(WSGIContext):
 
         return keys
 
+    def get_multiple_keys(self, env):
+        # get a list of keys from the keymaster containing one dict of keys for
+        # each of the keymaster root secret ids
+        keys = [self.get_keys(env)]
+        active_key_id = keys[0]['id']
+        for other_key_id in keys[0].get('all_ids', []):
+            if other_key_id == active_key_id:
+                continue
+            keys.append(self.get_keys(env, key_id=other_key_id))
+        return keys
+
 
 def dump_crypto_meta(crypto_meta):
     """
@@ -260,6 +274,8 @@ def append_crypto_meta(value, crypto_meta):
     :param crypto_meta: a dict of crypto meta
     :return: a string of the form <value>; swift_meta=<serialized crypto meta>
     """
+    if not isinstance(value, str):
+        raise ValueError
     return '%s; swift_meta=%s' % (value, dump_crypto_meta(crypto_meta))
 
 
