@@ -64,7 +64,7 @@ import os
 import re
 
 from swift.common.swob import Range
-from swift.common.utils import json, public
+from swift.common.utils import json, public, reiterate
 from swift.common.db import utf8encode
 
 from six.moves.urllib.parse import quote, urlparse
@@ -72,10 +72,10 @@ from six.moves.urllib.parse import quote, urlparse
 from swift.common.middleware.s3api.controllers.base import Controller, \
     bucket_operation, object_operation, check_container_existence
 from swift.common.middleware.s3api.s3response import InvalidArgument, \
-    ErrorResponse, MalformedXML, BadDigest, \
+    ErrorResponse, MalformedXML, \
     InvalidPart, BucketAlreadyExists, EntityTooSmall, InvalidPartOrder, \
     InvalidRequest, HTTPOk, HTTPNoContent, NoSuchKey, NoSuchUpload, \
-    NoSuchBucket
+    NoSuchBucket, BucketAlreadyOwnedByYou
 from swift.common.middleware.s3api.exception import BadSwiftRequest
 from swift.common.middleware.s3api.utils import unique_id, \
     MULTIUPLOAD_SUFFIX, S3Timestamp, sysmeta_header
@@ -365,7 +365,7 @@ class UploadsController(Controller):
 
         try:
             req.get_response(self.app, 'PUT', container, '')
-        except BucketAlreadyExists:
+        except (BucketAlreadyExists, BucketAlreadyOwnedByYou):
             pass
 
         obj = '%s/%s' % (req.object_name, upload_id)
@@ -540,8 +540,8 @@ class UploadController(Controller):
         """
         upload_id = req.params['uploadId']
         resp = _get_upload_info(req, self.app, upload_id)
-        headers = {}
-        for key, val in resp.headers.iteritems():
+        headers = {'Accept': 'application/json'}
+        for key, val in resp.headers.items():
             _key = key.lower()
             if _key.startswith('x-amz-meta-'):
                 headers['x-object-meta-' + _key[11:]] = val
@@ -577,21 +577,13 @@ class UploadController(Controller):
                           'etag': o['hash'],
                           'size_bytes': o['bytes']}) for o in objinfo)
 
+        s3_etag_hasher = md5()
         manifest = []
         previous_number = 0
         try:
             xml = req.xml(MAX_COMPLETE_UPLOAD_BODY_SIZE)
             if not xml:
                 raise InvalidRequest(msg='You must specify at least one part')
-            if 'content-md5' in req.headers:
-                # If an MD5 was provided, we need to verify it.
-                # Note that S3Request already took care of translating to ETag
-                if req.headers['etag'] != md5(xml).hexdigest():
-                    raise BadDigest(content_md5=req.headers['content-md5'])
-                # We're only interested in the body here, in the
-                # multipart-upload controller -- *don't* let it get
-                # plumbed down to the object-server
-                del req.headers['etag']
 
             complete_elem = fromstring(
                 xml, 'CompleteMultipartUpload', self.logger)
@@ -613,9 +605,11 @@ class UploadController(Controller):
                     raise InvalidPart(upload_id=upload_id,
                                       part_number=part_number)
 
+                s3_etag_hasher.update(etag.decode('hex'))
                 info['size_bytes'] = int(info['size_bytes'])
                 manifest.append(info)
         except (XMLSyntaxError, DocumentInvalid):
+            # NB: our schema definitions catch uploads with no parts here
             raise MalformedXML()
         except ErrorResponse:
             raise
@@ -623,63 +617,106 @@ class UploadController(Controller):
             self.logger.error(e)
             raise
 
+        s3_etag = '%s-%d' % (s3_etag_hasher.hexdigest(), len(manifest))
+        headers[sysmeta_header('object', 'etag')] = s3_etag
+        # Leave base header value blank; SLO will populate
+        c_etag = '; s3_etag=%s' % s3_etag
+        headers['X-Object-Sysmeta-Container-Update-Override-Etag'] = c_etag
+
         # Check the size of each segment except the last and make sure they are
         # all more than the minimum upload chunk size
         for info in manifest[:-1]:
             if info['size_bytes'] < self.conf.min_segment_size:
                 raise EntityTooSmall()
 
-        try:
-            # TODO: add support for versioning
-            if manifest:
-                resp = req.get_response(self.app, 'PUT',
-                                        body=json.dumps(manifest),
-                                        query={'multipart-manifest': 'put'},
-                                        headers=headers)
-            else:
-                # the upload must have consisted of a single zero-length part
-                # just write it directly
-                resp = req.get_response(self.app, 'PUT', body='',
-                                        headers=headers)
-        except BadSwiftRequest as e:
-            msg = str(e)
-            expected_msg = 'too small; each segment must be at least 1 byte'
-            if expected_msg in msg:
-                # FIXME: AWS S3 allows a smaller object than 5 MB if there is
-                # only one part.  Use a COPY request to copy the part object
-                # from the segments container instead.
-                raise EntityTooSmall(msg)
-            else:
-                raise
+        def response_iter():
+            # NB: XML requires that the XML declaration, if present, be at the
+            # very start of the document. Clients *will* call us out on not
+            # being valid XML if we pass through whitespace before it.
+            # Track whether we've sent anything yet so we can yield out that
+            # declaration *first*
+            yielded_anything = False
 
-        # clean up the multipart-upload record
-        obj = '%s/%s' % (req.object_name, upload_id)
-        try:
-            req.get_response(self.app, 'DELETE', container, obj)
-        except NoSuchKey:
-            pass  # We know that this existed long enough for us to HEAD
+            try:
+                try:
+                    # TODO: add support for versioning
+                    put_resp = req.get_response(
+                        self.app, 'PUT', body=json.dumps(manifest),
+                        query={'multipart-manifest': 'put',
+                               'heartbeat': 'on'},
+                        headers=headers)
+                    if put_resp.status_int == 202:
+                        body = []
+                        put_resp.fix_conditional_response()
+                        for chunk in put_resp.response_iter:
+                            if not chunk.strip():
+                                if not yielded_anything:
+                                    yield ('<?xml version="1.0" '
+                                           'encoding="UTF-8"?>\n')
+                                yielded_anything = True
+                                yield chunk
+                            body.append(chunk)
+                        body = json.loads(''.join(body))
+                        if body['Response Status'] != '201 Created':
+                            raise InvalidRequest(
+                                status=body['Response Status'],
+                                msg='\n'.join(': '.join(err)
+                                              for err in body['Errors']))
+                except BadSwiftRequest as e:
+                    msg = str(e)
+                    expected_msg = ('too small; each segment must be '
+                                    'at least 1 byte')
+                    if expected_msg in msg:
+                        # FIXME: AWS S3 allows a smaller object than 5 MB if
+                        # there is only one part.  Use a COPY request to copy
+                        # the part object from the segments container instead.
+                        raise EntityTooSmall(msg)
+                    else:
+                        raise
 
-        result_elem = Element('CompleteMultipartUploadResult')
+                # clean up the multipart-upload record
+                obj = '%s/%s' % (req.object_name, upload_id)
+                try:
+                    req.get_response(self.app, 'DELETE', container, obj)
+                except NoSuchKey:
+                    # We know that this existed long enough for us to HEAD
+                    pass
 
-        # NOTE: boto with sig v4 appends port to HTTP_HOST value at the
-        # request header when the port is non default value and it makes
-        # req.host_url like as http://localhost:8080:8080/path
-        # that obviously invalid. Probably it should be resolved at
-        # swift.common.swob though, tentatively we are parsing and
-        # reconstructing the correct host_url info here.
-        # in detail, https://github.com/boto/boto/pull/3513
-        parsed_url = urlparse(req.host_url)
-        host_url = '%s://%s' % (parsed_url.scheme, parsed_url.hostname)
-        if parsed_url.port:
-            host_url += ':%s' % parsed_url.port
+                result_elem = Element('CompleteMultipartUploadResult')
 
-        SubElement(result_elem, 'Location').text = host_url + req.path
-        SubElement(result_elem, 'Bucket').text = req.container_name
-        SubElement(result_elem, 'Key').text = req.object_name
-        SubElement(result_elem, 'ETag').text = resp.etag
+                # NOTE: boto with sig v4 appends port to HTTP_HOST value at
+                # the request header when the port is non default value and it
+                # makes req.host_url like as http://localhost:8080:8080/path
+                # that obviously invalid. Probably it should be resolved at
+                # swift.common.swob though, tentatively we are parsing and
+                # reconstructing the correct host_url info here.
+                # in detail, https://github.com/boto/boto/pull/3513
+                parsed_url = urlparse(req.host_url)
+                host_url = '%s://%s' % (parsed_url.scheme, parsed_url.hostname)
+                if parsed_url.port:
+                    host_url += ':%s' % parsed_url.port
 
-        resp.body = tostring(result_elem)
-        resp.status = 200
+                SubElement(result_elem, 'Location').text = host_url + req.path
+                SubElement(result_elem, 'Bucket').text = req.container_name
+                SubElement(result_elem, 'Key').text = req.object_name
+                SubElement(result_elem, 'ETag').text = '"%s"' % s3_etag
+                resp.headers.pop('ETag', None)
+                if yielded_anything:
+                    yield '\n'
+                yield tostring(result_elem,
+                               xml_declaration=not yielded_anything)
+            except ErrorResponse as err_resp:
+                if yielded_anything:
+                    err_resp.xml_declaration = False
+                    yield '\n'
+                else:
+                    # Oh good, we can still change HTTP status code, too!
+                    resp.status = err_resp.status
+                for chunk in err_resp({}, lambda *a: None):
+                    yield chunk
+
+        resp = HTTPOk()  # assume we're good for now... but see above!
+        resp.app_iter = reiterate(response_iter())
         resp.content_type = "application/xml"
 
         return resp
