@@ -14,7 +14,7 @@
 # limitations under the License.
 
 import base64
-from collections import defaultdict
+from collections import defaultdict, OrderedDict
 from email.header import Header
 from hashlib import sha1, sha256, md5
 import hmac
@@ -24,7 +24,7 @@ import six
 from six.moves.urllib.parse import quote, unquote, parse_qsl
 import string
 
-from swift.common.utils import split_path, close_if_possible
+from swift.common.utils import split_path, json, get_swift_info
 from swift.common import swob
 from swift.common.http import HTTP_OK, HTTP_CREATED, HTTP_ACCEPTED, \
     HTTP_NO_CONTENT, HTTP_UNAUTHORIZED, HTTP_FORBIDDEN, HTTP_NOT_FOUND, \
@@ -44,7 +44,7 @@ from swift.common.middleware.s3api.controllers import ServiceController, \
     UploadController, UploadsController, VersioningController, \
     UnsupportedController, S3AclController, BucketController
 from swift.common.middleware.s3api.s3response import AccessDenied, \
-    InvalidArgument, InvalidDigest, \
+    InvalidArgument, InvalidDigest, BucketAlreadyOwnedByYou, \
     RequestTimeTooSkewed, S3Response, SignatureDoesNotMatch, \
     BucketAlreadyExists, BucketNotEmpty, EntityTooLarge, \
     InternalError, NoSuchBucket, NoSuchKey, PreconditionFailed, InvalidRange, \
@@ -110,43 +110,16 @@ def _header_acl_property(resource):
                     doc='Get and set the %s acl property' % resource)
 
 
-class HashingInput(object):
-    """
-    wsgi.input wrapper to verify the hash of the input as it's read.
-    """
-    def __init__(self, reader, content_length, hasher, expected_hex_hash):
-        self._input = reader
-        self._to_read = content_length
-        self._hasher = hasher()
-        self._expected = expected_hex_hash
-
-    def read(self, size=None):
-        chunk = self._input.read(size)
-        self._hasher.update(chunk)
-        self._to_read -= len(chunk)
-        if self._to_read < 0 or (size > len(chunk) and self._to_read) or (
-                self._to_read == 0 and
-                self._hasher.hexdigest() != self._expected):
-            self.close()
-            # Since we don't return the last chunk, the PUT never completes
-            raise swob.HTTPUnprocessableEntity(
-                'The X-Amz-Content-SHA56 you specified did not match '
-                'what we received.')
-        return chunk
-
-    def close(self):
-        close_if_possible(self._input)
-
-
 class SigV4Mixin(object):
     """
     A request class mixin to provide S3 signature v4 functionality
     """
 
     def check_signature(self, secret):
+        secret = utf8encode(secret)
         user_signature = self.signature
         derived_secret = 'AWS4' + secret
-        for scope_piece in self.scope:
+        for scope_piece in self.scope.values():
             derived_secret = hmac.new(
                 derived_secret, scope_piece, sha256).digest()
         valid_signature = hmac.new(
@@ -204,6 +177,8 @@ class SigV4Mixin(object):
         err = None
         try:
             expires = int(self.params['X-Amz-Expires'])
+        except KeyError:
+            raise AccessDenied()
         except ValueError:
             err = 'X-Amz-Expires should be a number'
         else:
@@ -221,6 +196,15 @@ class SigV4Mixin(object):
         if int(self.timestamp) + expires < S3Timestamp.now():
             raise AccessDenied('Request has expired')
 
+    def _parse_credential(self, credential_string):
+        parts = credential_string.split("/")
+        # credential must be in following format:
+        # <access-key-id>/<date>/<AWS-region>/<AWS-service>/aws4_request
+        if not parts[0] or len(parts) != 5:
+            raise AccessDenied()
+        return dict(zip(['access', 'date', 'region', 'service', 'terminal'],
+                        parts))
+
     def _parse_query_authentication(self):
         """
         Parse v4 query authentication
@@ -233,10 +217,11 @@ class SigV4Mixin(object):
             raise InvalidArgument('X-Amz-Algorithm',
                                   self.params.get('X-Amz-Algorithm'))
         try:
-            cred_param = self.params['X-Amz-Credential'].split("/")
-            access = cred_param[0]
+            cred_param = self._parse_credential(
+                self.params['X-Amz-Credential'])
             sig = self.params['X-Amz-Signature']
-            expires = self.params['X-Amz-Expires']
+            if not sig:
+                raise AccessDenied()
         except KeyError:
             raise AccessDenied()
 
@@ -248,12 +233,26 @@ class SigV4Mixin(object):
 
         self._signed_headers = set(signed_headers.split(';'))
 
-        # credential must be in following format:
-        # <access-key-id>/<date>/<AWS-region>/<AWS-service>/aws4_request
-        if not all([access, sig, len(cred_param) == 5, expires]):
-            raise AccessDenied()
+        invalid_messages = {
+            'date': 'Invalid credential date "%s". This date is not the same '
+                    'as X-Amz-Date: "%s".',
+            'region': "Error parsing the X-Amz-Credential parameter; "
+                    "the region '%s' is wrong; expecting '%s'",
+            'service': 'Error parsing the X-Amz-Credential parameter; '
+                    'incorrect service "%s". This endpoint belongs to "%s".',
+            'terminal': 'Error parsing the X-Amz-Credential parameter; '
+                    'incorrect terminal "%s". This endpoint uses "%s".',
+        }
+        for key in ('date', 'region', 'service', 'terminal'):
+            if cred_param[key] != self.scope[key]:
+                kwargs = {}
+                if key == 'region':
+                    kwargs = {'region': self.scope['region']}
+                raise AuthorizationQueryParametersError(
+                    invalid_messages[key] % (cred_param[key], self.scope[key]),
+                    **kwargs)
 
-        return access, sig
+        return cred_param['access'], sig
 
     def _parse_header_authentication(self):
         """
@@ -265,23 +264,39 @@ class SigV4Mixin(object):
         """
 
         auth_str = self.headers['Authorization']
-        cred_param = auth_str.partition(
-            "Credential=")[2].split(',')[0].split("/")
-        access = cred_param[0]
+        cred_param = self._parse_credential(auth_str.partition(
+            "Credential=")[2].split(',')[0])
         sig = auth_str.partition("Signature=")[2].split(',')[0]
+        if not sig:
+            raise AccessDenied()
         signed_headers = auth_str.partition(
             "SignedHeaders=")[2].split(',', 1)[0]
-        # credential must be in following format:
-        # <access-key-id>/<date>/<AWS-region>/<AWS-service>/aws4_request
-        if not all([access, sig, len(cred_param) == 5]):
-            raise AccessDenied()
         if not signed_headers:
             # TODO: make sure if is it Malformed?
             raise AuthorizationHeaderMalformed()
 
+        invalid_messages = {
+            'date': 'Invalid credential date "%s". This date is not the same '
+                    'as X-Amz-Date: "%s".',
+            'region': "The authorization header is malformed; the region '%s' "
+                    "is wrong; expecting '%s'",
+            'service': 'The authorization header is malformed; incorrect '
+                    'service "%s". This endpoint belongs to "%s".',
+            'terminal': 'The authorization header is malformed; incorrect '
+                    'terminal "%s". This endpoint uses "%s".',
+        }
+        for key in ('date', 'region', 'service', 'terminal'):
+            if cred_param[key] != self.scope[key]:
+                kwargs = {}
+                if key == 'region':
+                    kwargs = {'region': self.scope['region']}
+                raise AuthorizationHeaderMalformed(
+                    invalid_messages[key] % (cred_param[key], self.scope[key]),
+                    **kwargs)
+
         self._signed_headers = set(signed_headers.split(';'))
 
-        return access, sig
+        return cred_param['access'], sig
 
     def _canonical_query_string(self):
         return '&'.join(
@@ -386,28 +401,17 @@ class SigV4Mixin(object):
             raise InvalidRequest(msg)
         else:
             hashed_payload = self.headers['X-Amz-Content-SHA256']
-            if hashed_payload != 'UNSIGNED-PAYLOAD':
-                if self.content_length == 0:
-                    if hashed_payload != sha256().hexdigest():
-                        raise BadDigest(
-                            'The X-Amz-Content-SHA56 you specified did not '
-                            'match what we received.')
-                elif self.content_length:
-                    self.environ['wsgi.input'] = HashingInput(
-                        self.environ['wsgi.input'],
-                        self.content_length,
-                        sha256,
-                        hashed_payload)
-                # else, length not provided -- Swift will kick out a
-                # 411 Length Required which will get translated back
-                # to a S3-style response in S3Request._swift_error_codes
         cr.append(hashed_payload)
         return '\n'.join(cr).encode('utf-8')
 
     @property
     def scope(self):
-        return [self.timestamp.amz_date_format.split('T')[0],
-                self.location, SERVICE, 'aws4_request']
+        return OrderedDict([
+            ('date', self.timestamp.amz_date_format.split('T')[0]),
+            ('region', self.location),
+            ('service', SERVICE),
+            ('terminal', 'aws4_request'),
+        ])
 
     def _string_to_sign(self):
         """
@@ -415,7 +419,7 @@ class SigV4Mixin(object):
         """
         return '\n'.join(['AWS4-HMAC-SHA256',
                           self.timestamp.amz_date_format,
-                          '/'.join(self.scope),
+                          '/'.join(self.scope.values()),
                           sha256(self._canonical_request()).hexdigest()])
 
     def signature_does_not_match_kwargs(self):
@@ -457,8 +461,8 @@ class S3Request(swob.Request):
     bucket_acl = _header_acl_property('container')
     object_acl = _header_acl_property('object')
 
-    def __init__(self, env, app=None, slo_enabled=True,
-                 storage_domain='', location='US', force_request_log=False,
+    def __init__(self, env, app=None, slo_enabled=True, storage_domain='',
+                 location='us-east-1', force_request_log=False,
                  dns_compliant_bucket_names=True, allow_multipart_uploads=True,
                  allow_no_owner=False):
         # NOTE: app and allow_no_owner are not used by this class, need for
@@ -501,6 +505,7 @@ class S3Request(swob.Request):
         self.environ['swift.leave_relative_location'] = True
 
     def check_signature(self, secret):
+        secret = utf8encode(secret)
         user_signature = self.signature
         valid_signature = base64.b64encode(hmac.new(
             secret, self.string_to_sign, sha1).digest()).strip()
@@ -630,8 +635,10 @@ class S3Request(swob.Request):
         :raises: NotS3Request
         """
         if self._is_query_auth:
+            self._validate_expire_param()
             return self._parse_query_authentication()
         elif self._is_header_auth:
+            self._validate_dates()
             return self._parse_header_authentication()
         else:
             # if this request is neither query auth nor header auth
@@ -646,7 +653,7 @@ class S3Request(swob.Request):
         # Expires header is a float since epoch
         try:
             ex = S3Timestamp(float(self.params['Expires']))
-        except ValueError:
+        except (KeyError, ValueError):
             raise AccessDenied()
 
         if S3Timestamp.now() > ex:
@@ -663,11 +670,6 @@ class S3Request(swob.Request):
         :raises: AccessDenied
         :raises: RequestTimeTooSkewed
         """
-        if self._is_query_auth:
-            self._validate_expire_param()
-            # TODO: make sure the case if timestamp param in query
-            return
-
         date_header = self.headers.get('Date')
         amz_date_header = self.headers.get('X-Amz-Date')
         if not date_header and not amz_date_header:
@@ -695,8 +697,6 @@ class S3Request(swob.Request):
                 raise InvalidArgument('Content-Length',
                                       self.environ['CONTENT_LENGTH'])
 
-        self._validate_dates()
-
         value = _header_strip(self.headers.get('Content-MD5'))
         if value is not None:
             if not re.match('^[A-Za-z0-9+/]+={0,2}$', value):
@@ -741,8 +741,17 @@ class S3Request(swob.Request):
         if 'x-amz-mfa' in self.headers:
             raise S3NotImplemented('MFA Delete is not supported.')
 
-        if 'x-amz-server-side-encryption' in self.headers:
-            raise S3NotImplemented('Server-side encryption is not supported.')
+        sse_value = self.headers.get('x-amz-server-side-encryption')
+        if sse_value is not None:
+            if sse_value not in ('aws:kms', 'AES256'):
+                raise InvalidArgument(
+                    'x-amz-server-side-encryption', sse_value,
+                    'The encryption method specified is not supported')
+            encryption_enabled = get_swift_info(admin=True)['admin'].get(
+                'encryption', {}).get('enabled')
+            if not encryption_enabled or sse_value != 'AES256':
+                raise S3NotImplemented(
+                    'Server-side encryption is not supported.')
 
         if 'x-amz-website-redirect-location' in self.headers:
             raise S3NotImplemented('Website redirection is not supported.')
@@ -1053,6 +1062,22 @@ class S3Request(swob.Request):
             env['HTTP_X_COPY_FROM'] = env['HTTP_X_AMZ_COPY_SOURCE']
             del env['HTTP_X_AMZ_COPY_SOURCE']
             env['CONTENT_LENGTH'] = '0'
+            # Content type cannot be modified on COPY
+            env.pop('CONTENT_TYPE', None)
+            if env.pop('HTTP_X_AMZ_METADATA_DIRECTIVE', None) == 'REPLACE':
+                env['HTTP_X_FRESH_METADATA'] = 'True'
+            else:
+                copy_exclude_headers = ('HTTP_CONTENT_DISPOSITION',
+                                        'HTTP_CONTENT_ENCODING',
+                                        'HTTP_CONTENT_LANGUAGE',
+                                        'HTTP_EXPIRES',
+                                        'HTTP_CACHE_CONTROL',
+                                        'HTTP_X_ROBOTS_TAG')
+                for key in copy_exclude_headers:
+                    env.pop(key, None)
+                for key in list(env.keys()):
+                    if key.startswith('HTTP_X_OBJECT_META_'):
+                        del env[key]
 
         if self.force_request_log:
             env['swift.proxy_access_log_made'] = False
@@ -1130,6 +1155,7 @@ class S3Request(swob.Request):
                 ],
                 'PUT': [
                     HTTP_CREATED,
+                    HTTP_ACCEPTED,  # For SLO with heartbeating
                 ],
                 'POST': [
                     HTTP_ACCEPTED,
@@ -1142,6 +1168,20 @@ class S3Request(swob.Request):
 
         return code_map[method]
 
+    def _bucket_put_accepted_error(self, container, app):
+        sw_req = self.to_swift_req('HEAD', container, None)
+        info = get_container_info(sw_req.environ, app)
+        sysmeta = info.get('sysmeta', {})
+        try:
+            acl = json.loads(sysmeta.get('s3api-acl',
+                                         sysmeta.get('swift3-acl', '{}')))
+            owner = acl.get('Owner')
+        except (ValueError, TypeError, KeyError):
+            owner = None
+        if owner is None or owner == self.user_id:
+            raise BucketAlreadyOwnedByYou(container)
+        raise BucketAlreadyExists(container)
+
     def _swift_error_codes(self, method, container, obj, env, app):
         """
         Returns a dict from expected Swift error codes to the corresponding S3
@@ -1163,7 +1203,8 @@ class S3Request(swob.Request):
                     HTTP_NOT_FOUND: (NoSuchBucket, container),
                 },
                 'PUT': {
-                    HTTP_ACCEPTED: (BucketAlreadyExists, container),
+                    HTTP_ACCEPTED: (self._bucket_put_accepted_error, container,
+                                    app),
                 },
                 'POST': {
                     HTTP_NOT_FOUND: (NoSuchBucket, container),
@@ -1232,15 +1273,12 @@ class S3Request(swob.Request):
         sw_req = self.to_swift_req(method, container, obj, headers=headers,
                                    body=body, query=query)
 
-        try:
-            sw_resp = sw_req.get_response(app)
-        except swob.HTTPException as err:
-            sw_resp = err
-        else:
-            # reuse account and tokens
-            _, self.account, _ = split_path(sw_resp.environ['PATH_INFO'],
-                                            2, 3, True)
-            self.account = utf8encode(self.account)
+        sw_resp = sw_req.get_response(app)
+
+        # reuse account and tokens
+        _, self.account, _ = split_path(sw_resp.environ['PATH_INFO'],
+                                        2, 3, True)
+        self.account = utf8encode(self.account)
 
         resp = S3Response.from_swift_resp(sw_resp)
         status = resp.status_int  # pylint: disable-msg=E1101
@@ -1369,8 +1407,8 @@ class S3AclRequest(S3Request):
     """
     S3Acl request object.
     """
-    def __init__(self, env, app, slo_enabled=True,
-                 storage_domain='', location='US', force_request_log=False,
+    def __init__(self, env, app, slo_enabled=True, storage_domain='',
+                 location='us-east-1', force_request_log=False,
                  dns_compliant_bucket_names=True, allow_multipart_uploads=True,
                  allow_no_owner=False):
         super(S3AclRequest, self).__init__(
