@@ -294,12 +294,12 @@ class SymlinkContainerContext(WSGIContext):
         :return: modified json body
         """
         with closing_if_possible(resp_iter):
-            resp_body = ''.join(resp_iter)
+            resp_body = b''.join(resp_iter)
         body_json = json.loads(resp_body)
         swift_version, account, _junk = split_path(req.path, 2, 3, True)
         new_body = json.dumps(
             [self._extract_symlink_path_json(obj_dict, swift_version, account)
-             for obj_dict in body_json])
+             for obj_dict in body_json]).encode('ascii')
         self.update_content_length(len(new_body))
         return [new_body]
 
