@@ -193,6 +193,14 @@ def get_socket(conf):
     bind_timeout = int(conf.get('bind_timeout', 30))
     retry_until = time.time() + bind_timeout
     warn_ssl = False
+
+    try:
+        keepidle = int(conf.get('keep_idle', 600))
+        if keepidle <= 0 or keepidle >= 2 ** 15 - 1:
+            raise ValueError()
+    except (ValueError, KeyError, TypeError):
+        raise ConfigFileError()
+
     while not sock and time.time() < retry_until:
         try:
             sock = listen(bind_addr, backlog=int(conf.get('backlog', 4096)),
@@ -214,7 +222,7 @@ def get_socket(conf):
     sock.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)
     sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
     if hasattr(socket, 'TCP_KEEPIDLE'):
-        sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_KEEPIDLE, 600)
+        sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_KEEPIDLE, keepidle)
     if warn_ssl:
         ssl_warning_message = _('WARNING: SSL should only be enabled for '
                                 'testing purposes. Use external SSL '
@@ -432,8 +440,13 @@ class SwiftHttpProtocol(wsgi.HttpProtocol):
         """
         Redirect logging other messages by the underlying WSGI software.
         """
-        logger = getattr(self.server.app, 'logger', None) or self.server.log
-        logger.error('ERROR WSGI: ' + f, *a)
+        logger = getattr(self.server.app, 'logger', None)
+        if logger:
+            logger.error('ERROR WSGI: ' + f, *a)
+        else:
+            # eventlet<=0.17.4 doesn't have an error method, and in newer
+            # versions the output from error is same as info anyway
+            self.server.log.info('ERROR WSGI: ' + f, *a)
 
 
 class SwiftHttpProxiedProtocol(SwiftHttpProtocol):
@@ -447,6 +460,10 @@ class SwiftHttpProxiedProtocol(SwiftHttpProtocol):
     See http://www.haproxy.org/download/1.7/doc/proxy-protocol.txt for
     protocol details.
     """
+    def __init__(self, *a, **kw):
+        self.proxy_address = None
+        SwiftHttpProtocol.__init__(self, *a, **kw)
+
     def handle_error(self, connection_line):
         if not six.PY2:
             connection_line = connection_line.decode('latin-1')
@@ -476,35 +493,45 @@ class SwiftHttpProxiedProtocol(SwiftHttpProtocol):
         # additional wrapping further pollutes the raw socket.
         connection_line = self.rfile.readline(self.server.url_length_limit)
 
-        if connection_line.startswith(b'PROXY'):
-            proxy_parts = connection_line.split(b' ')
-            if len(proxy_parts) >= 2 and proxy_parts[0] == b'PROXY':
-                if proxy_parts[1] in (b'TCP4', b'TCP6') and \
-                        len(proxy_parts) == 6:
-                    if six.PY2:
-                        self.client_address = (proxy_parts[2], proxy_parts[4])
-                    else:
-                        self.client_address = (
-                            proxy_parts[2].decode('latin-1'),
-                            proxy_parts[4].decode('latin-1'))
-                elif proxy_parts[1].startswith(b'UNKNOWN'):
-                    # "UNKNOWN", in PROXY protocol version 1, means "not
-                    # TCP4 or TCP6". This includes completely legitimate
-                    # things like QUIC or Unix domain sockets. The PROXY
-                    # protocol (section 2.1) states that the receiver
-                    # (that's us) MUST ignore anything after "UNKNOWN" and
-                    # before the CRLF, essentially discarding the first
-                    # line.
-                    pass
-                else:
-                    self.handle_error(connection_line)
+        if not connection_line.startswith(b'PROXY '):
+            return self.handle_error(connection_line)
+
+        proxy_parts = connection_line.strip(b'\r\n').split(b' ')
+        if proxy_parts[1].startswith(b'UNKNOWN'):
+            # "UNKNOWN", in PROXY protocol version 1, means "not
+            # TCP4 or TCP6". This includes completely legitimate
+            # things like QUIC or Unix domain sockets. The PROXY
+            # protocol (section 2.1) states that the receiver
+            # (that's us) MUST ignore anything after "UNKNOWN" and
+            # before the CRLF, essentially discarding the first
+            # line.
+            pass
+        elif proxy_parts[1] in (b'TCP4', b'TCP6') and len(proxy_parts) == 6:
+            if six.PY2:
+                self.client_address = (proxy_parts[2], proxy_parts[4])
+                self.proxy_address = (proxy_parts[3], proxy_parts[5])
             else:
-                self.handle_error(connection_line)
+                self.client_address = (
+                    proxy_parts[2].decode('latin-1'),
+                    proxy_parts[4].decode('latin-1'))
+                self.proxy_address = (
+                    proxy_parts[3].decode('latin-1'),
+                    proxy_parts[5].decode('latin-1'))
         else:
             self.handle_error(connection_line)
 
         return SwiftHttpProtocol.handle(self)
 
+    def get_environ(self):
+        environ = SwiftHttpProtocol.get_environ(self)
+        if self.proxy_address:
+            environ['SERVER_ADDR'] = self.proxy_address[0]
+            environ['SERVER_PORT'] = self.proxy_address[1]
+            if self.proxy_address[1] == '443':
+                environ['wsgi.url_scheme'] = 'https'
+                environ['HTTPS'] = 'on'
+        return environ
+
 
 def run_server(conf, logger, sock, global_conf=None):
     # Ensure TZ environment variable exists to avoid stat('/etc/localtime') on
