@@ -97,13 +97,31 @@ class DiskFileWriter(object):
 
     :param fs: internal file system object to use
     :param name: standard object name
-    :param fp: `StringIO` in-memory representation object
     """
-    def __init__(self, fs, name, fp):
+    def __init__(self, fs, name):
         self._filesystem = fs
         self._name = name
-        self._fp = fp
+        self._fp = None
         self._upload_size = 0
+        self._chunks_etag = hashlib.md5()
+
+    def open(self):
+        """
+        Prepare to accept writes.
+
+        Create a new ``StringIO`` object for a started-but-not-yet-finished
+        PUT.
+        """
+        self._fp = moves.cStringIO()
+        return self
+
+    def close(self):
+        """
+        Clean up resources following an ``open()``.
+
+        Note: If ``put()`` has not been called, the data written will be lost.
+        """
+        self._fp = None
 
     def write(self, chunk):
         """
@@ -113,7 +131,15 @@ class DiskFileWriter(object):
         """
         self._fp.write(chunk)
         self._upload_size += len(chunk)
-        return self._upload_size
+        self._chunks_etag.update(chunk)
+
+    def chunks_finished(self):
+        """
+        Expose internal stats about written chunks.
+
+        :returns: a tuple, (upload_size, etag)
+        """
+        return self._upload_size, self._chunks_etag.hexdigest()
 
     def put(self, metadata):
         """
@@ -273,11 +299,14 @@ class DiskFile(object):
         self._filesystem = fs
         self.fragments = None
 
-    def open(self, modernize=False):
+    def open(self, modernize=False, current_time=None):
         """
         Open the file and read the metadata.
 
         This method must populate the _metadata attribute.
+
+        :param current_time: Unix time used in checking expiration. If not
+             present, the current time will be used.
         :raises DiskFileCollision: on name mis-match with metadata
         :raises DiskFileDeleted: if it does not exist, or a tombstone is
                                  present
@@ -287,7 +316,7 @@ class DiskFile(object):
         fp, self._metadata = self._filesystem.get_object(self._name)
         if fp is None:
             raise DiskFileDeleted()
-        self._fp = self._verify_data_file(fp)
+        self._fp = self._verify_data_file(fp, current_time)
         self._metadata = self._metadata or {}
         return self
 
@@ -313,7 +342,7 @@ class DiskFile(object):
         self._filesystem.del_object(name)
         return DiskFileQuarantined(msg)
 
-    def _verify_data_file(self, fp):
+    def _verify_data_file(self, fp, current_time):
         """
         Verify the metadata's name value matches what we think the object is
         named.
@@ -344,7 +373,9 @@ class DiskFile(object):
                 self._name, "bad metadata x-delete-at value %s" % (
                     self._metadata['X-Delete-At']))
         else:
-            if x_delete_at <= time.time():
+            if current_time is None:
+                current_time = time.time()
+            if x_delete_at <= current_time:
                 raise DiskFileNotExist('Expired')
         try:
             metadata_size = int(self._metadata['Content-Length'])
@@ -381,13 +412,15 @@ class DiskFile(object):
             raise DiskFileNotOpen()
         return self._metadata
 
-    def read_metadata(self):
+    def read_metadata(self, current_time=None):
         """
         Return the metadata for an object.
 
+        :param current_time: Unix time used in checking expiration. If not
+             present, the current time will be used.
         :returns: metadata dictionary for an object
         """
-        with self.open():
+        with self.open(current_time=current_time):
             return self.get_metadata()
 
     def reader(self, keep_cache=False):
@@ -406,6 +439,9 @@ class DiskFile(object):
         self._fp = None
         return dr
 
+    def writer(self, size=None):
+        return DiskFileWriter(self._filesystem, self._name)
+
     @contextmanager
     def create(self, size=None):
         """
@@ -416,11 +452,11 @@ class DiskFile(object):
                      disk
         :raises DiskFileNoSpace: if a size is specified and allocation fails
         """
-        fp = moves.cStringIO()
+        writer = self.writer(size)
         try:
-            yield DiskFileWriter(self._filesystem, self._name, fp)
+            yield writer.open()
         finally:
-            del fp
+            writer.close()
 
     def write_metadata(self, metadata):
         """
