@@ -15,7 +15,6 @@
 
 import json
 import errno
-import math
 import os
 from os.path import join
 import random
@@ -32,8 +31,10 @@ from eventlet.support.greenlets import GreenletExit
 from swift import gettext_ as _
 from swift.common.utils import (
     whataremyips, unlink_older_than, compute_eta, get_logger,
-    dump_recon_cache, mkdirs, config_true_value, list_from_csv,
-    tpool_reraise, GreenAsyncPile, Timestamp, remove_file)
+    dump_recon_cache, mkdirs, config_true_value,
+    GreenAsyncPile, Timestamp, remove_file,
+    load_recon_cache, parse_override_options, distribute_evenly,
+    PrefixLoggerAdapter, remove_directory)
 from swift.common.header_key_dict import HeaderKeyDict
 from swift.common.bufferedhttp import http_connect
 from swift.common.daemon import Daemon
@@ -90,30 +91,6 @@ def _full_path(node, part, relative_path, policy):
         }
 
 
-def parse_override_options(**kwargs):
-    """
-    Return a dict with keys `override_devices` and `override_partitions` whose
-    values have been parsed from `kwargs`. If either key is found in `kwargs`
-    then copy its value from kwargs. Otherwise, if `once` is set in `kwargs`
-    then parse `devices` and `partitions` keys for the value of
-    `override_devices` and `override_partitions` respectively.
-
-    :return: a dict with keys `override_devices` and `override_partitions`
-    """
-    if kwargs.get('once', False):
-        devices = list_from_csv(kwargs.get('devices'))
-        partitions = [
-            int(p) for p in list_from_csv(kwargs.get('partitions'))]
-    else:
-        devices = []
-        partitions = []
-
-    return {
-        'override_devices': kwargs.get('override_devices', devices),
-        'override_partitions': kwargs.get('override_partitions', partitions),
-    }
-
-
 class RebuildingECDiskFileStream(object):
     """
     This class wraps the reconstructed fragment archive data and
@@ -166,8 +143,8 @@ class ObjectReconstructor(Daemon):
         :param logger: logging object
         """
         self.conf = conf
-        self.logger = logger or get_logger(
-            conf, log_route='object-reconstructor')
+        self.logger = PrefixLoggerAdapter(
+            logger or get_logger(conf, log_route='object-reconstructor'), {})
         self.devices_dir = conf.get('devices', '/srv/node')
         self.mount_check = config_true_value(conf.get('mount_check', 'true'))
         self.swift_dir = conf.get('swift_dir', '/etc/swift')
@@ -188,6 +165,12 @@ class ObjectReconstructor(Daemon):
         self.partition_times = []
         self.interval = int(conf.get('interval') or
                             conf.get('run_pause') or 30)
+        if 'run_pause' in conf and 'interval' not in conf:
+            self.logger.warning('Option object-reconstructor/run_pause '
+                                'is deprecated and will be removed in a '
+                                'future version. Update your configuration'
+                                ' to use option object-reconstructor/'
+                                'interval.')
         self.http_timeout = int(conf.get('http_timeout', 60))
         self.lockup_timeout = int(conf.get('lockup_timeout', 1800))
         self.recon_cache_path = conf.get('recon_cache_path',
@@ -236,29 +219,34 @@ class ObjectReconstructor(Daemon):
         """
         if self.reconstructor_workers < 1:
             return
-        override_options = parse_override_options(once=once, **kwargs)
+        override_opts = parse_override_options(once=once, **kwargs)
 
         # Note that this get re-used when dumping stats and in is_healthy
         self.all_local_devices = self.get_local_devices()
 
-        if override_options['override_devices']:
-            devices = [d for d in override_options['override_devices']
+        if override_opts.devices:
+            devices = [d for d in override_opts.devices
                        if d in self.all_local_devices]
         else:
             devices = list(self.all_local_devices)
         if not devices:
             # we only need a single worker to do nothing until a ring change
-            yield dict(override_options)
+            yield dict(override_devices=override_opts.devices,
+                       override_partitions=override_opts.partitions,
+                       multiprocess_worker_index=0)
             return
-        # for somewhat uniform load per worker use same max_devices_per_worker
-        # when handling all devices or just override devices...
-        max_devices_per_worker = int(math.ceil(
-            1.0 * len(self.all_local_devices) / self.reconstructor_workers))
-        # ...but only use enough workers for the actual devices being handled
-        n = int(math.ceil(1.0 * len(devices) / max_devices_per_worker))
-        override_devices_per_worker = [devices[i::n] for i in range(n)]
-        for override_devices in override_devices_per_worker:
-            yield dict(override_options, override_devices=override_devices)
+
+        # for somewhat uniform load per worker use same
+        # max_devices_per_worker when handling all devices or just override
+        # devices, but only use enough workers for the actual devices being
+        # handled
+        self.reconstructor_workers = min(self.reconstructor_workers,
+                                         len(devices))
+        for index, ods in enumerate(distribute_evenly(
+                devices, self.reconstructor_workers)):
+            yield dict(override_partitions=override_opts.partitions,
+                       override_devices=ods,
+                       multiprocess_worker_index=index)
 
     def is_healthy(self):
         """
@@ -276,14 +264,7 @@ class ObjectReconstructor(Daemon):
         """
         Aggregate per-disk rcache updates from child workers.
         """
-        try:
-            with open(self.rcache) as f:
-                existing_data = json.load(f)
-        except IOError as e:
-            if e.errno != errno.ENOENT:
-                raise
-            # dump_recon_cache will create new file and dirs
-            existing_data = {}
+        existing_data = load_recon_cache(self.rcache)
         first_start = time.time()
         last_finish = 0
         all_devices_reporting = True
@@ -602,6 +583,12 @@ class ObjectReconstructor(Daemon):
                 _("Nothing reconstructed for %s seconds."),
                 (time.time() - self.start))
 
+    def _emplace_log_prefix(self, worker_index):
+        self.logger.set_prefix("[worker %d/%d pid=%s] " % (
+            worker_index + 1,  # use 1-based indexing for more readable logs
+            self.reconstructor_workers,
+            os.getpid()))
+
     def kill_coros(self):
         """Utility function that kills all coroutines currently running."""
         for coro in list(self.run_pool.coroutines_running):
@@ -635,7 +622,7 @@ class ObjectReconstructor(Daemon):
     def _get_hashes(self, device, partition, policy, recalculate=None,
                     do_listdir=False):
         df_mgr = self._df_router[policy]
-        hashed, suffix_hashes = tpool_reraise(
+        hashed, suffix_hashes = tpool.execute(
             df_mgr._get_hashes, device, partition, policy,
             recalculate=recalculate, do_listdir=do_listdir)
         self.logger.update_stats('suffix.hashes', hashed)
@@ -666,13 +653,15 @@ class ObjectReconstructor(Daemon):
         return suffixes
 
     def rehash_remote(self, node, job, suffixes):
+        headers = self.headers.copy()
+        headers['X-Backend-Storage-Policy-Index'] = int(job['policy'])
         try:
             with Timeout(self.http_timeout):
                 conn = http_connect(
                     node['replication_ip'], node['replication_port'],
                     node['device'], job['partition'], 'REPLICATE',
                     '/' + '-'.join(sorted(suffixes)),
-                    headers=self.headers)
+                    headers=headers)
                 conn.getresponse().read()
         except (Exception, Timeout):
             self.logger.exception(
@@ -693,12 +682,14 @@ class ObjectReconstructor(Daemon):
         """
         # get hashes from the remote node
         remote_suffixes = None
+        headers = self.headers.copy()
+        headers['X-Backend-Storage-Policy-Index'] = int(job['policy'])
         try:
             with Timeout(self.http_timeout):
                 resp = http_connect(
                     node['replication_ip'], node['replication_port'],
                     node['device'], job['partition'], 'REPLICATE',
-                    '', headers=self.headers).getresponse()
+                    '', headers=headers).getresponse()
             if resp.status == HTTP_INSUFFICIENT_STORAGE:
                 self.logger.error(
                     _('%s responded as unmounted'),
@@ -754,6 +745,7 @@ class ObjectReconstructor(Daemon):
         :param frag_index: (int) the fragment index of data files to be deleted
         """
         df_mgr = self._df_router[job['policy']]
+        suffixes_to_delete = set()
         for object_hash, timestamps in objects.items():
             try:
                 df = df_mgr.get_diskfile_from_hash(
@@ -765,6 +757,10 @@ class ObjectReconstructor(Daemon):
                 self.logger.exception(
                     'Unable to purge DiskFile (%r %r %r)',
                     object_hash, timestamps['ts_data'], frag_index)
+            suffixes_to_delete.add(object_hash[-3:])
+
+        for suffix in suffixes_to_delete:
+            remove_directory(os.path.join(job['path'], suffix))
 
     def process_job(self, job):
         """
@@ -787,7 +783,6 @@ class ObjectReconstructor(Daemon):
 
         :param: the job dict, with the keys defined in ``_get_job_info``
         """
-        self.headers['X-Backend-Storage-Policy-Index'] = int(job['policy'])
         begin = time.time()
         if job['job_type'] == REVERT:
             self._revert(job, begin)
@@ -1029,9 +1024,10 @@ class ObjectReconstructor(Daemon):
     def get_local_devices(self):
         """Returns a set of all local devices in all EC policies."""
         policy2devices = self.get_policy2devices()
-        return reduce(set.union, (
-            set(d['device'] for d in devices)
-            for devices in policy2devices.values()), set())
+        local_devices = set()
+        for devices in policy2devices.values():
+            local_devices.update(d['device'] for d in devices)
+        return local_devices
 
     def collect_parts(self, override_devices=None, override_partitions=None):
         """
@@ -1199,7 +1195,7 @@ class ObjectReconstructor(Daemon):
             with Timeout(self.lockup_timeout):
                 self.run_pool.waitall()
         except (Exception, Timeout):
-            self.logger.exception(_("Exception in top-level"
+            self.logger.exception(_("Exception in top-level "
                                     "reconstruction loop"))
             self.kill_coros()
         finally:
@@ -1244,35 +1240,48 @@ class ObjectReconstructor(Daemon):
             recon_update['object_reconstruction_per_disk'] = {}
         dump_recon_cache(recon_update, self.rcache, self.logger)
 
-    def run_once(self, *args, **kwargs):
+    def post_multiprocess_run(self):
+        # This method is called after run_once when using multiple workers.
+        self.aggregate_recon_update()
+
+    def run_once(self, multiprocess_worker_index=None, *args, **kwargs):
+        if multiprocess_worker_index is not None:
+            self._emplace_log_prefix(multiprocess_worker_index)
         start = time.time()
         self.logger.info(_("Running object reconstructor in script mode."))
-        override_options = parse_override_options(once=True, **kwargs)
-        self.reconstruct(**override_options)
+        override_opts = parse_override_options(once=True, **kwargs)
+        self.reconstruct(override_devices=override_opts.devices,
+                         override_partitions=override_opts.partitions)
         total = (time.time() - start) / 60
         self.logger.info(
             _("Object reconstruction complete (once). (%.02f minutes)"), total)
         # Only dump stats if they would actually be meaningful -- i.e. we're
         # collecting per-disk stats and covering all partitions, or we're
         # covering all partitions, all disks.
-        if not override_options['override_partitions'] and (
-                self.reconstructor_workers > 0 or
-                not override_options['override_devices']):
-            self.final_recon_dump(total, **override_options)
+        if not override_opts.partitions and (
+                self.reconstructor_workers > 0 or not override_opts.devices):
+            self.final_recon_dump(
+                total, override_devices=override_opts.devices,
+                override_partitions=override_opts.partitions)
 
-    def run_forever(self, *args, **kwargs):
+    def run_forever(self, multiprocess_worker_index=None, *args, **kwargs):
+        if multiprocess_worker_index is not None:
+            self._emplace_log_prefix(multiprocess_worker_index)
         self.logger.info(_("Starting object reconstructor in daemon mode."))
         # Run the reconstructor continually
         while True:
             start = time.time()
             self.logger.info(_("Starting object reconstruction pass."))
-            override_options = parse_override_options(**kwargs)
+            override_opts = parse_override_options(**kwargs)
             # Run the reconstructor
-            self.reconstruct(**override_options)
+            self.reconstruct(override_devices=override_opts.devices,
+                             override_partitions=override_opts.partitions)
             total = (time.time() - start) / 60
             self.logger.info(
                 _("Object reconstruction complete. (%.02f minutes)"), total)
-            self.final_recon_dump(total, **override_options)
+            self.final_recon_dump(
+                total, override_devices=override_opts.devices,
+                override_partitions=override_opts.partitions)
             self.logger.debug('reconstruction sleeping for %s seconds.',
                               self.interval)
             sleep(self.interval)
