@@ -49,7 +49,7 @@ from swift.common.utils import Timestamp, config_true_value, \
 from swift.common.bufferedhttp import http_connect
 from swift.common import constraints
 from swift.common.exceptions import ChunkReadTimeout, ChunkWriteTimeout, \
-    ConnectionTimeout, RangeAlreadyComplete, ShortReadError
+    ConnectionTimeout, RangeAlreadyComplete
 from swift.common.header_key_dict import HeaderKeyDict
 from swift.common.http import is_informational, is_success, is_redirection, \
     is_server_error, HTTP_OK, HTTP_PARTIAL_CONTENT, HTTP_MULTIPLE_CHOICES, \
@@ -79,7 +79,7 @@ def update_headers(response, headers):
     if hasattr(headers, 'items'):
         headers = headers.items()
     for name, value in headers:
-        if name == 'etag':
+        if name.lower() == 'etag':
             response.headers[name] = value.replace('"', '')
         elif name.lower() not in (
                 'date', 'content-length', 'content-type',
@@ -199,7 +199,7 @@ def headers_to_object_info(headers, status_int=HTTP_OK):
     """
     headers, meta, sysmeta = _prep_headers_to_info(headers, 'object')
     transient_sysmeta = {}
-    for key, val in six.iteritems(headers):
+    for key, val in headers.items():
         if is_object_transient_sysmeta(key):
             key = strip_object_transient_sysmeta_prefix(key.lower())
             transient_sysmeta[key] = val
@@ -590,7 +590,8 @@ def _get_info_from_memcache(app, env, account, container=None):
     memcache = getattr(app, 'memcache', None) or env.get('swift.cache')
     if memcache:
         info = memcache.get(cache_key)
-        if info:
+        if info and six.PY2:
+            # Get back to native strings
             for key in info:
                 if isinstance(info[key], six.text_type):
                     info[key] = info[key].encode("utf-8")
@@ -598,6 +599,7 @@ def _get_info_from_memcache(app, env, account, container=None):
                     for subkey, value in info[key].items():
                         if isinstance(value, six.text_type):
                             info[key][subkey] = value.encode("utf-8")
+        if info:
             env.setdefault('swift.infocache', {})[cache_key] = info
         return info
     return None
@@ -748,37 +750,6 @@ def bytes_to_skip(record_size, range_start):
     return (record_size - (range_start % record_size)) % record_size
 
 
-class ByteCountEnforcer(object):
-    """
-    Enforces that successive calls to file_like.read() give at least
-    <nbytes> bytes before exhaustion.
-
-    If file_like fails to do so, ShortReadError is raised.
-
-    If more than <nbytes> bytes are read, we don't care.
-    """
-
-    def __init__(self, file_like, nbytes):
-        """
-        :param file_like: file-like object
-        :param nbytes: number of bytes expected, or None if length is unknown.
-        """
-        self.file_like = file_like
-        self.nbytes = self.bytes_left = nbytes
-
-    def read(self, amt=None):
-        chunk = self.file_like.read(amt)
-        if self.bytes_left is None:
-            return chunk
-        elif len(chunk) == 0 and self.bytes_left > 0:
-            raise ShortReadError(
-                "Too few bytes; read %d, expecting %d" % (
-                    self.nbytes - self.bytes_left, self.nbytes))
-        else:
-            self.bytes_left -= len(chunk)
-            return chunk
-
-
 class ResumingGetter(object):
     def __init__(self, app, req, server_type, node_iter, partition, path,
                  backend_headers, concurrency=1, client_chunk_size=None,
@@ -976,9 +947,9 @@ class ResumingGetter(object):
                     except ChunkReadTimeout:
                         new_source, new_node = self._get_source_and_node()
                         if new_source:
-                            self.app.error_occurred(
-                                node[0], _('Trying to read object during '
-                                           'GET (retrying)'))
+                            self.app.exception_occurred(
+                                node[0], _('Object'),
+                                _('Trying to read during GET (retrying)'))
                             # Close-out the connection as best as possible.
                             if getattr(source[0], 'swift_conn', None):
                                 close_swift_conn(source[0])
@@ -992,21 +963,16 @@ class ResumingGetter(object):
                         else:
                             raise StopIteration()
 
-            def iter_bytes_from_response_part(part_file, nbytes):
+            def iter_bytes_from_response_part(part_file):
                 nchunks = 0
-                buf = ''
-                part_file = ByteCountEnforcer(part_file, nbytes)
+                buf = b''
                 while True:
                     try:
                         with ChunkReadTimeout(node_timeout):
                             chunk = part_file.read(self.app.object_chunk_size)
                             nchunks += 1
-                            # NB: this append must be *inside* the context
-                            # manager for test.unit.SlowBody to do its thing
                             buf += chunk
-                            if nbytes is not None:
-                                nbytes -= len(chunk)
-                    except (ChunkReadTimeout, ShortReadError):
+                    except ChunkReadTimeout:
                         exc_type, exc_value, exc_traceback = exc_info()
                         if self.newest or self.server_type != 'Object':
                             raise
@@ -1016,12 +982,12 @@ class ResumingGetter(object):
                             six.reraise(exc_type, exc_value, exc_traceback)
                         except RangeAlreadyComplete:
                             break
-                        buf = ''
+                        buf = b''
                         new_source, new_node = self._get_source_and_node()
                         if new_source:
-                            self.app.error_occurred(
-                                node[0], _('Trying to read object during '
-                                           'GET (retrying)'))
+                            self.app.exception_occurred(
+                                node[0], _('Object'),
+                                _('Trying to read during GET (retrying)'))
                             # Close-out the connection as best as possible.
                             if getattr(source[0], 'swift_conn', None):
                                 close_swift_conn(source[0])
@@ -1040,9 +1006,8 @@ class ResumingGetter(object):
                             except StopIteration:
                                 # Tried to find a new node from which to
                                 # finish the GET, but failed. There's
-                                # nothing more we can do here.
-                                six.reraise(exc_type, exc_value, exc_traceback)
-                            part_file = ByteCountEnforcer(part_file, nbytes)
+                                # nothing more to do here.
+                                return
                         else:
                             six.reraise(exc_type, exc_value, exc_traceback)
                     else:
@@ -1054,7 +1019,7 @@ class ResumingGetter(object):
                             else:
                                 self.skip_bytes -= len(buf)
                                 self.bytes_used_from_backend += len(buf)
-                                buf = ''
+                                buf = b''
 
                         if not chunk:
                             if buf:
@@ -1062,7 +1027,7 @@ class ResumingGetter(object):
                                         self.app.client_timeout):
                                     self.bytes_used_from_backend += len(buf)
                                     yield buf
-                                buf = ''
+                                buf = b''
                             break
 
                         if client_chunk_size is not None:
@@ -1078,7 +1043,7 @@ class ResumingGetter(object):
                             with ChunkWriteTimeout(self.app.client_timeout):
                                 self.bytes_used_from_backend += len(buf)
                                 yield buf
-                            buf = ''
+                            buf = b''
 
                         # This is for fairness; if the network is outpacing
                         # the CPU, we'll always be able to read and write
@@ -1104,18 +1069,10 @@ class ResumingGetter(object):
                 while True:
                     start_byte, end_byte, length, headers, part = \
                         get_next_doc_part()
-                    # note: learn_size_from_content_range() sets
-                    # self.skip_bytes
                     self.learn_size_from_content_range(
                         start_byte, end_byte, length)
                     self.bytes_used_from_backend = 0
-                    # not length; that refers to the whole object, so is the
-                    # wrong value to use for GET-range responses
-                    byte_count = ((end_byte - start_byte + 1) - self.skip_bytes
-                                  if (end_byte is not None
-                                      and start_byte is not None)
-                                  else None)
-                    part_iter = iter_bytes_from_response_part(part, byte_count)
+                    part_iter = iter_bytes_from_response_part(part)
                     yield {'start_byte': start_byte, 'end_byte': end_byte,
                            'entity_length': length, 'headers': headers,
                            'part_iter': part_iter}
