@@ -32,8 +32,7 @@ from swift.common.middleware.s3api.utils import mktime
 
 from test.functional.s3api import S3ApiBase
 from test.functional.s3api.s3_test_client import Connection
-from test.functional.s3api.utils import get_error_code, get_error_msg, \
-    calculate_md5
+from test.functional.s3api.utils import get_error_code, get_error_msg
 
 
 def setUpModule():
@@ -68,7 +67,7 @@ class TestS3ApiMultiUpload(S3ApiBase):
         self.conn.make_request('PUT', bucket)
         query = 'uploads'
         for key, key_headers in izip_longest(keys, headers):
-            for i in xrange(trials):
+            for i in range(trials):
                 status, resp_headers, body = \
                     self.conn.make_request('POST', bucket, key,
                                            headers=key_headers, query=query)
@@ -304,8 +303,14 @@ class TestS3ApiMultiUpload(S3ApiBase):
         self.assertCommonResponseHeaders(headers)
         self.assertTrue('content-type' in headers)
         self.assertEqual(headers['content-type'], 'application/xml')
-        self.assertTrue('content-length' in headers)
-        self.assertEqual(headers['content-length'], str(len(body)))
+        if 'content-length' in headers:
+            self.assertEqual(headers['content-length'], str(len(body)))
+        else:
+            self.assertIn('transfer-encoding', headers)
+            self.assertEqual(headers['transfer-encoding'], 'chunked')
+        lines = body.split('\n')
+        self.assertTrue(lines[0].startswith('<?xml'), body)
+        self.assertTrue(lines[0].endswith('?>'), body)
         elem = fromstring(body, 'CompleteMultipartUploadResult')
         # TODO: use tf.config value
         self.assertEqual(
@@ -313,8 +318,65 @@ class TestS3ApiMultiUpload(S3ApiBase):
             elem.find('Location').text)
         self.assertEqual(elem.find('Bucket').text, bucket)
         self.assertEqual(elem.find('Key').text, key)
-        # TODO: confirm completed etag value
-        self.assertTrue(elem.find('ETag').text is not None)
+        concatted_etags = ''.join(etag.strip('"') for etag in etags)
+        exp_etag = '"%s-%s"' % (
+            md5(concatted_etags.decode('hex')).hexdigest(), len(etags))
+        etag = elem.find('ETag').text
+        self.assertEqual(etag, exp_etag)
+
+        exp_size = self.min_segment_size * len(etags)
+        swift_etag = '"%s"' % md5(concatted_etags).hexdigest()
+        # TODO: GET via swift api, check against swift_etag
+
+        # Check object
+        def check_obj(req_headers, exp_status):
+            status, headers, body = \
+                self.conn.make_request('HEAD', bucket, key, req_headers)
+            self.assertEqual(status, exp_status)
+            self.assertCommonResponseHeaders(headers)
+            self.assertIn('content-length', headers)
+            if exp_status == 412:
+                self.assertNotIn('etag', headers)
+                self.assertEqual(headers['content-length'], '0')
+            else:
+                self.assertIn('etag', headers)
+                self.assertEqual(headers['etag'], exp_etag)
+                if exp_status == 304:
+                    self.assertEqual(headers['content-length'], '0')
+                else:
+                    self.assertEqual(headers['content-length'], str(exp_size))
+
+        check_obj({}, 200)
+
+        # Sanity check conditionals
+        check_obj({'If-Match': 'some other thing'}, 412)
+        check_obj({'If-None-Match': 'some other thing'}, 200)
+
+        # More interesting conditional cases
+        check_obj({'If-Match': exp_etag}, 200)
+        check_obj({'If-Match': swift_etag}, 412)
+        check_obj({'If-None-Match': swift_etag}, 200)
+        check_obj({'If-None-Match': exp_etag}, 304)
+
+        # Check listings
+        status, headers, body = self.conn.make_request('GET', bucket)
+        self.assertEqual(status, 200)
+
+        elem = fromstring(body, 'ListBucketResult')
+        resp_objects = elem.findall('./Contents')
+        self.assertEqual(len(list(resp_objects)), 1)
+        for o in resp_objects:
+            self.assertEqual(o.find('Key').text, key)
+            self.assertIsNotNone(o.find('LastModified').text)
+            self.assertRegexpMatches(
+                o.find('LastModified').text,
+                r'^\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}\.\d{3}Z$')
+            self.assertEqual(o.find('ETag').text, exp_etag)
+            self.assertEqual(o.find('Size').text, str(exp_size))
+            self.assertIsNotNone(o.find('StorageClass').text is not None)
+            self.assertEqual(o.find('Owner/ID').text, self.conn.user_id)
+            self.assertEqual(o.find('Owner/DisplayName').text,
+                             self.conn.user_id)
 
     def test_initiate_multi_upload_error(self):
         bucket = 'bucket'
@@ -491,7 +553,7 @@ class TestS3ApiMultiUpload(S3ApiBase):
         upload_id = elem.find('UploadId').text
 
         etags = []
-        for i in xrange(1, 3):
+        for i in range(1, 3):
             query = 'partNumber=%s&uploadId=%s' % (i, upload_id)
             status, headers, body = \
                 self.conn.make_request('PUT', bucket, keys[0], query=query)
@@ -565,7 +627,7 @@ class TestS3ApiMultiUpload(S3ApiBase):
 
         # multi parts with no body
         etags = []
-        for i in xrange(1, 3):
+        for i in range(1, 3):
             query = 'partNumber=%s&uploadId=%s' % (i, upload_id)
             status, headers, body = \
                 self.conn.make_request('PUT', bucket, key, query=query)
@@ -580,7 +642,7 @@ class TestS3ApiMultiUpload(S3ApiBase):
 
         # multi parts with all parts less than min segment size
         etags = []
-        for i in xrange(1, 3):
+        for i in range(1, 3):
             query = 'partNumber=%s&uploadId=%s' % (i, upload_id)
             status, headers, body = \
                 self.conn.make_request('PUT', bucket, key, query=query,
@@ -619,7 +681,7 @@ class TestS3ApiMultiUpload(S3ApiBase):
 
         etags = []
         body_size = [self.min_segment_size, self.min_segment_size - 1, 2]
-        for i in xrange(1, 3):
+        for i in range(1, 3):
             query = 'partNumber=%s&uploadId=%s' % (i, upload_id)
             status, headers, body = \
                 self.conn.make_request('PUT', bucket, key, query=query,
@@ -643,7 +705,7 @@ class TestS3ApiMultiUpload(S3ApiBase):
 
         etags = []
         body_size = [self.min_segment_size, self.min_segment_size, 2]
-        for i in xrange(1, 3):
+        for i in range(1, 3):
             query = 'partNumber=%s&uploadId=%s' % (i, upload_id)
             status, headers, body = \
                 self.conn.make_request('PUT', bucket, key, query=query,
@@ -668,7 +730,7 @@ class TestS3ApiMultiUpload(S3ApiBase):
         upload_id = elem.find('UploadId').text
 
         etags = []
-        for i in xrange(1, 4):
+        for i in range(1, 4):
             query = 'partNumber=%s&uploadId=%s' % (i, upload_id)
             status, headers, body = \
                 self.conn.make_request('PUT', bucket, key,
@@ -845,27 +907,6 @@ class TestS3ApiMultiUploadSigV4(TestS3ApiMultiUpload):
         self.assertEqual(status, 200)  # sanity
         self.assertEqual(content, body)  # sanity
 
-        # Can delete it with DeleteMultipleObjects request
-        elem = Element('Delete')
-        SubElement(elem, 'Quiet').text = 'true'
-        obj_elem = SubElement(elem, 'Object')
-        SubElement(obj_elem, 'Key').text = key
-        body = tostring(elem, use_s3ns=False)
-
-        status, headers, body = self.conn.make_request(
-            'POST', bucket, body=body, query='delete',
-            headers={'Content-MD5': calculate_md5(body)})
-        self.assertEqual(status, 200)
-        self.assertCommonResponseHeaders(headers)
-
-        status, headers, body = \
-            self.conn.make_request('GET', bucket, key)
-        self.assertEqual(status, 404)  # sanity
-
-        # Now we can delete
-        status, headers, body = \
-            self.conn.make_request('DELETE', bucket)
-        self.assertEqual(status, 204)  # sanity
 
 if __name__ == '__main__':
     unittest2.main()
