@@ -406,17 +406,17 @@ class FakeMemcache(object):
 
 
 def readuntil2crlfs(fd):
-    rv = ''
-    lc = ''
+    rv = b''
+    lc = b''
     crlfs = 0
     while crlfs < 2:
         c = fd.read(1)
         if not c:
             raise ValueError("didn't get two CRLFs; just got %r" % rv)
         rv = rv + c
-        if c == '\r' and lc != '\n':
+        if c == b'\r' and lc != b'\n':
             crlfs = 0
-        if lc == '\r' and c == '\n':
+        if lc == b'\r' and c == b'\n':
             crlfs += 1
         lc = c
     return rv
@@ -867,9 +867,6 @@ def fake_http_connect(*code_iter, **kwargs):
 
     class FakeConn(object):
 
-        SLOW_READS = 4
-        SLOW_WRITES = 4
-
         def __init__(self, status, etag=None, body='', timestamp='1',
                      headers=None, expect_headers=None, connection_id=None,
                      give_send=None, give_expect=None):
@@ -895,12 +892,6 @@ def fake_http_connect(*code_iter, **kwargs):
                     self._next_sleep = kwargs['slow'].pop(0)
                 except IndexError:
                     self._next_sleep = None
-
-            # if we're going to be slow, we need a body to send slowly
-            am_slow, _junk = self.get_slow()
-            if am_slow and len(self.body) < self.SLOW_READS:
-                self.body += " " * (self.SLOW_READS - len(self.body))
-
             # be nice to trixy bits with node_iter's
             eventlet.sleep()
 
@@ -936,7 +927,6 @@ def fake_http_connect(*code_iter, **kwargs):
                 else:
                     etag = '"68b329da9893e34099c7d8ad5cb9c940"'
 
-            am_slow, _junk = self.get_slow()
             headers = HeaderKeyDict({
                 'content-length': len(self.body),
                 'content-type': 'x-application/test',
@@ -959,6 +949,9 @@ def fake_http_connect(*code_iter, **kwargs):
                     headers['x-container-timestamp'] = '1'
             except StopIteration:
                 pass
+            am_slow, value = self.get_slow()
+            if am_slow:
+                headers['content-length'] = '4'
             headers.update(self.headers)
             return headers.items()
 
@@ -975,16 +968,12 @@ def fake_http_connect(*code_iter, **kwargs):
         def read(self, amt=None):
             am_slow, value = self.get_slow()
             if am_slow:
-                if self.sent < self.SLOW_READS:
-                    slowly_read_byte = self.body[self.sent]
+                if self.sent < 4:
                     self.sent += 1
                     eventlet.sleep(value)
-                    return slowly_read_byte
-            if amt is None:
-                rv = self.body[self.sent:]
-            else:
-                rv = self.body[self.sent:self.sent + amt]
-            self.sent += len(rv)
+                    return ' '
+            rv = self.body[:amt]
+            self.body = self.body[amt:]
             return rv
 
         def send(self, data=None):
@@ -992,7 +981,7 @@ def fake_http_connect(*code_iter, **kwargs):
                 self.give_send(self, data)
             am_slow, value = self.get_slow()
             if am_slow:
-                if self.received < self.SLOW_WRITES:
+                if self.received < 4:
                     self.received += 1
                     eventlet.sleep(value)
 
