@@ -14,7 +14,7 @@
 # limitations under the License.
 
 import base64
-from hashlib import md5
+import hashlib
 from mock import patch
 import os
 import time
@@ -28,43 +28,46 @@ from swift.common.utils import json
 from test.unit.common.middleware.s3api import S3ApiTestCase
 from test.unit.common.middleware.s3api.helpers import UnreadableInput
 from swift.common.middleware.s3api.etree import fromstring, tostring
-from swift.common.middleware.s3api.subresource import Owner, Grant, User, ACL, \
-    encode_acl, decode_acl, ACLPublicRead
+from swift.common.middleware.s3api.subresource import Owner, Grant, User, \
+    ACL, encode_acl, decode_acl, ACLPublicRead
 from test.unit.common.middleware.s3api.test_s3_acl import s3acl
 from swift.common.middleware.s3api.utils import sysmeta_header, mktime, \
     S3Timestamp
 from swift.common.middleware.s3api.s3request import MAX_32BIT_INT
-from swift.common.middleware.s3api.controllers.multi_upload import \
-    MULTIUPLOAD_SUFFIX
 
 xml = '<CompleteMultipartUpload>' \
     '<Part>' \
     '<PartNumber>1</PartNumber>' \
-    '<ETag>HASH</ETag>' \
+    '<ETag>0123456789abcdef</ETag>' \
     '</Part>' \
     '<Part>' \
     '<PartNumber>2</PartNumber>' \
-    '<ETag>"HASH"</ETag>' \
+    '<ETag>"fedcba9876543210"</ETag>' \
     '</Part>' \
     '</CompleteMultipartUpload>'
 
 objects_template = \
-    (('object/X/1', '2014-05-07T19:47:51.592270', 'HASH', 100),
-     ('object/X/2', '2014-05-07T19:47:52.592270', 'HASH', 200))
+    (('object/X/1', '2014-05-07T19:47:51.592270', '0123456789abcdef', 100),
+     ('object/X/2', '2014-05-07T19:47:52.592270', 'fedcba9876543210', 200))
 
 multiparts_template = \
     (('object/X', '2014-05-07T19:47:50.592270', 'HASH', 1),
-     ('object/X/1', '2014-05-07T19:47:51.592270', 'HASH', 11),
-     ('object/X/2', '2014-05-07T19:47:52.592270', 'HASH', 21),
+     ('object/X/1', '2014-05-07T19:47:51.592270', '0123456789abcdef', 11),
+     ('object/X/2', '2014-05-07T19:47:52.592270', 'fedcba9876543210', 21),
      ('object/Y', '2014-05-07T19:47:53.592270', 'HASH', 2),
-     ('object/Y/1', '2014-05-07T19:47:54.592270', 'HASH', 12),
-     ('object/Y/2', '2014-05-07T19:47:55.592270', 'HASH', 22),
+     ('object/Y/1', '2014-05-07T19:47:54.592270', '0123456789abcdef', 12),
+     ('object/Y/2', '2014-05-07T19:47:55.592270', 'fedcba9876543210', 22),
      ('object/Z', '2014-05-07T19:47:56.592270', 'HASH', 3),
-     ('object/Z/1', '2014-05-07T19:47:57.592270', 'HASH', 13),
-     ('object/Z/2', '2014-05-07T19:47:58.592270', 'HASH', 23),
+     ('object/Z/1', '2014-05-07T19:47:57.592270', '0123456789abcdef', 13),
+     ('object/Z/2', '2014-05-07T19:47:58.592270', 'fedcba9876543210', 23),
      ('subdir/object/Z', '2014-05-07T19:47:58.592270', 'HASH', 4),
-     ('subdir/object/Z/1', '2014-05-07T19:47:58.592270', 'HASH', 41),
-     ('subdir/object/Z/2', '2014-05-07T19:47:58.592270', 'HASH', 41))
+     ('subdir/object/Z/1', '2014-05-07T19:47:58.592270', '0123456789abcdef',
+      41),
+     ('subdir/object/Z/2', '2014-05-07T19:47:58.592270', 'fedcba9876543210',
+      41))
+
+s3_etag = '"%s-2"' % hashlib.md5(
+    '0123456789abcdeffedcba9876543210'.decode('hex')).hexdigest()
 
 
 class TestS3ApiMultiUpload(S3ApiTestCase):
@@ -152,6 +155,8 @@ class TestS3ApiMultiUpload(S3ApiTestCase):
                                      'Date': self.get_date_header()})
         status, headers, body = self.call_s3api(req)
         self.assertEqual(self._get_error_code(body), 'InvalidRequest')
+        self.assertEqual(self._get_error_message(body),
+                         'A key must be specified')
 
     @s3acl
     def test_bucket_multipart_uploads_complete(self):
@@ -549,11 +554,14 @@ class TestS3ApiMultiUpload(S3ApiTestCase):
         self.assertEqual(req_headers.get('X-Object-Meta-Foo'), 'bar')
         self.assertNotIn('Etag', req_headers)
         self.assertNotIn('Content-MD5', req_headers)
-        method, path, _ = self.swift.calls_with_headers[-2]
-        self.assertEqual(method, 'PUT')
-        self.assertEqual(
-            path,
-            '/v1/AUTH_test/bucket%s' % MULTIUPLOAD_SUFFIX)
+        self.assertEqual([
+            ('HEAD', '/v1/AUTH_test/bucket'),
+            ('PUT', '/v1/AUTH_test/bucket+segments'),
+            ('HEAD', '/v1/AUTH_test'),
+            ('HEAD', '/v1/AUTH_test/bucket+segments'),
+            ('PUT', '/v1/AUTH_test/bucket+segments/object/X'),
+        ], self.swift.calls)
+        self.swift.clear_calls()
 
     def test_object_multipart_upload_initiate(self):
         self._test_object_multipart_upload_initiate({})
@@ -658,20 +666,127 @@ class TestS3ApiMultiUpload(S3ApiTestCase):
         self.assertEqual(self._get_error_code(body), 'NoSuchBucket')
 
     def test_object_multipart_upload_complete(self):
-        content_md5 = base64.b64encode(md5(xml).digest())
         req = Request.blank('/bucket/object?uploadId=X',
                             environ={'REQUEST_METHOD': 'POST'},
                             headers={'Authorization': 'AWS test:tester:hmac',
-                                     'Date': self.get_date_header(),
-                                     'Content-MD5': content_md5, },
+                                     'Date': self.get_date_header(), },
                             body=xml)
         status, headers, body = self.call_s3api(req)
-        fromstring(body, 'CompleteMultipartUploadResult')
+        elem = fromstring(body, 'CompleteMultipartUploadResult')
+        self.assertNotIn('Etag', headers)
+        self.assertEqual(elem.find('ETag').text, s3_etag)
         self.assertEqual(status.split()[0], '200')
 
+        self.assertEqual(self.swift.calls, [
+            # Bucket exists
+            ('HEAD', '/v1/AUTH_test/bucket'),
+            # Segment container exists
+            ('HEAD', '/v1/AUTH_test/bucket+segments/object/X'),
+            # Get the currently-uploaded segments
+            ('GET', '/v1/AUTH_test/bucket+segments?delimiter=/'
+                    '&format=json&prefix=object/X/'),
+            # Create the SLO
+            ('PUT', '/v1/AUTH_test/bucket/object'
+                    '?heartbeat=on&multipart-manifest=put'),
+            # Delete the in-progress-upload marker
+            ('DELETE', '/v1/AUTH_test/bucket+segments/object/X')
+        ])
+
         _, _, headers = self.swift.calls_with_headers[-2]
         self.assertEqual(headers.get('X-Object-Meta-Foo'), 'bar')
         self.assertEqual(headers.get('Content-Type'), 'baz/quux')
+        # SLO will provide a base value
+        override_etag = '; s3_etag=%s' % s3_etag.strip('"')
+        h = 'X-Object-Sysmeta-Container-Update-Override-Etag'
+        self.assertEqual(headers.get(h), override_etag)
+
+    def test_object_multipart_upload_complete_with_heartbeat(self):
+        self.swift.register(
+            'HEAD', '/v1/AUTH_test/bucket+segments/heartbeat-ok/X',
+            swob.HTTPOk, {}, None)
+        self.swift.register(
+            'GET', '/v1/AUTH_test/bucket+segments', swob.HTTPOk, {},
+            json.dumps([
+                {'name': item[0].replace('object', 'heartbeat-ok'),
+                 'last_modified': item[1], 'hash': item[2], 'bytes': item[3]}
+                for item in objects_template
+            ]))
+        self.swift.register(
+            'PUT', '/v1/AUTH_test/bucket/heartbeat-ok',
+            swob.HTTPAccepted, {}, [' ', ' ', ' ', json.dumps({
+                'Etag': '"slo-etag"',
+                'Response Status': '201 Created',
+                'Errors': [],
+            })])
+        self.swift.register(
+            'DELETE', '/v1/AUTH_test/bucket+segments/heartbeat-ok/X',
+            swob.HTTPNoContent, {}, None)
+
+        req = Request.blank('/bucket/heartbeat-ok?uploadId=X',
+                            environ={'REQUEST_METHOD': 'POST'},
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header(), },
+                            body=xml)
+        status, headers, body = self.call_s3api(req)
+        lines = body.split('\n')
+        self.assertTrue(lines[0].startswith('<?xml '))
+        self.assertTrue(lines[1])
+        self.assertFalse(lines[1].strip())
+        fromstring(body, 'CompleteMultipartUploadResult')
+        self.assertEqual(status.split()[0], '200')
+        # NB: s3_etag includes quotes
+        self.assertIn('<ETag>%s</ETag>' % s3_etag, body)
+        self.assertEqual(self.swift.calls, [
+            ('HEAD', '/v1/AUTH_test/bucket'),
+            ('HEAD', '/v1/AUTH_test/bucket+segments/heartbeat-ok/X'),
+            ('GET', '/v1/AUTH_test/bucket+segments?'
+                    'delimiter=/&format=json&prefix=heartbeat-ok/X/'),
+            ('PUT', '/v1/AUTH_test/bucket/heartbeat-ok?'
+                    'heartbeat=on&multipart-manifest=put'),
+            ('DELETE', '/v1/AUTH_test/bucket+segments/heartbeat-ok/X'),
+        ])
+
+    def test_object_multipart_upload_complete_failure_with_heartbeat(self):
+        self.swift.register(
+            'HEAD', '/v1/AUTH_test/bucket+segments/heartbeat-fail/X',
+            swob.HTTPOk, {}, None)
+        self.swift.register(
+            'GET', '/v1/AUTH_test/bucket+segments', swob.HTTPOk, {},
+            json.dumps([
+                {'name': item[0].replace('object', 'heartbeat-fail'),
+                 'last_modified': item[1], 'hash': item[2], 'bytes': item[3]}
+                for item in objects_template
+            ]))
+        self.swift.register(
+            'PUT', '/v1/AUTH_test/bucket/heartbeat-fail',
+            swob.HTTPAccepted, {}, [' ', ' ', ' ', json.dumps({
+                'Response Status': '400 Bad Request',
+                'Errors': [['some/object', '404 Not Found']],
+            })])
+
+        req = Request.blank('/bucket/heartbeat-fail?uploadId=X',
+                            environ={'REQUEST_METHOD': 'POST'},
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header(), },
+                            body=xml)
+        status, headers, body = self.call_s3api(req)
+        lines = body.split('\n')
+        self.assertTrue(lines[0].startswith('<?xml '))
+        self.assertTrue(lines[1])
+        self.assertFalse(lines[1].strip())
+        fromstring(body, 'Error')
+        self.assertEqual(status.split()[0], '200')
+        self.assertEqual(self._get_error_code(body), 'InvalidRequest')
+        self.assertEqual(self._get_error_message(body),
+                         'some/object: 404 Not Found')
+        self.assertEqual(self.swift.calls, [
+            ('HEAD', '/v1/AUTH_test/bucket'),
+            ('HEAD', '/v1/AUTH_test/bucket+segments/heartbeat-fail/X'),
+            ('GET', '/v1/AUTH_test/bucket+segments?'
+                    'delimiter=/&format=json&prefix=heartbeat-fail/X/'),
+            ('PUT', '/v1/AUTH_test/bucket/heartbeat-fail?'
+                    'heartbeat=on&multipart-manifest=put'),
+        ])
 
     def test_object_multipart_upload_complete_404_on_marker_delete(self):
         segment_bucket = '/v1/AUTH_test/bucket+segments'
@@ -774,6 +889,48 @@ class TestS3ApiMultiUpload(S3ApiTestCase):
                          'allowed object size.')
         self.assertNotIn('PUT', [method for method, _ in self.swift.calls])
 
+    def test_object_multipart_upload_complete_zero_segments(self):
+        segment_bucket = '/v1/AUTH_test/empty-bucket+segments'
+
+        object_list = [{
+            'name': 'object/X/1',
+            'last_modified': self.last_modified,
+            'hash': 'd41d8cd98f00b204e9800998ecf8427e',
+            'bytes': '0',
+        }]
+
+        self.swift.register('GET', segment_bucket, swob.HTTPOk, {},
+                            json.dumps(object_list))
+        self.swift.register('HEAD', '/v1/AUTH_test/empty-bucket',
+                            swob.HTTPNoContent, {}, None)
+        self.swift.register('HEAD', segment_bucket + '/object/X',
+                            swob.HTTPOk, {'x-object-meta-foo': 'bar',
+                                          'content-type': 'baz/quux'}, None)
+        self.swift.register('PUT', '/v1/AUTH_test/empty-bucket/object',
+                            swob.HTTPCreated, {}, None)
+        self.swift.register('DELETE', segment_bucket + '/object/X/1',
+                            swob.HTTPOk, {}, None)
+        self.swift.register('DELETE', segment_bucket + '/object/X',
+                            swob.HTTPOk, {}, None)
+
+        xml = '<CompleteMultipartUpload></CompleteMultipartUpload>'
+
+        req = Request.blank('/empty-bucket/object?uploadId=X',
+                            environ={'REQUEST_METHOD': 'POST'},
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header(), },
+                            body=xml)
+        status, headers, body = self.call_s3api(req)
+        self.assertEqual(status.split()[0], '400')
+        fromstring(body, 'Error')
+
+        self.assertEqual(self.swift.calls, [
+            ('HEAD', '/v1/AUTH_test/empty-bucket'),
+            ('HEAD', '/v1/AUTH_test/empty-bucket+segments/object/X'),
+            ('GET', '/v1/AUTH_test/empty-bucket+segments?delimiter=/&'
+                    'format=json&prefix=object/X/'),
+        ])
+
     def test_object_multipart_upload_complete_single_zero_length_segment(self):
         segment_bucket = '/v1/AUTH_test/empty-bucket+segments'
         put_headers = {'etag': self.etag, 'last-modified': self.last_modified}
@@ -820,8 +977,8 @@ class TestS3ApiMultiUpload(S3ApiTestCase):
             ('HEAD', '/v1/AUTH_test/empty-bucket+segments/object/X'),
             ('GET', '/v1/AUTH_test/empty-bucket+segments?delimiter=/&'
                     'format=json&prefix=object/X/'),
-            ('PUT',
-             '/v1/AUTH_test/empty-bucket/object?multipart-manifest=put'),
+            ('PUT', '/v1/AUTH_test/empty-bucket/object?'
+                    'heartbeat=on&multipart-manifest=put'),
             ('DELETE', '/v1/AUTH_test/empty-bucket+segments/object/X'),
         ])
         _, _, put_headers = self.swift.calls_with_headers[-2]
@@ -884,12 +1041,12 @@ class TestS3ApiMultiUpload(S3ApiTestCase):
         object_list = [{
             'name': 'object/X/1',
             'last_modified': self.last_modified,
-            'hash': 'some hash',
+            'hash': '0123456789abcdef0123456789abcdef',
             'bytes': '100',
         }, {
             'name': 'object/X/2',
             'last_modified': self.last_modified,
-            'hash': 'some other hash',
+            'hash': 'fedcba9876543210fedcba9876543210',
             'bytes': '1',
         }, {
             'name': 'object/X/3',
@@ -911,11 +1068,11 @@ class TestS3ApiMultiUpload(S3ApiTestCase):
         xml = '<CompleteMultipartUpload>' \
             '<Part>' \
             '<PartNumber>1</PartNumber>' \
-            '<ETag>some hash</ETag>' \
+            '<ETag>0123456789abcdef0123456789abcdef</ETag>' \
             '</Part>' \
             '<Part>' \
             '<PartNumber>2</PartNumber>' \
-            '<ETag>some other hash</ETag>' \
+            '<ETag>fedcba9876543210fedcba9876543210</ETag>' \
             '</Part>' \
             '<Part>' \
             '<PartNumber>3</PartNumber>' \
@@ -930,16 +1087,28 @@ class TestS3ApiMultiUpload(S3ApiTestCase):
                             body=xml)
         status, headers, body = self.call_s3api(req)
         self.assertEqual(status.split()[0], '200')
+        elem = fromstring(body, 'CompleteMultipartUploadResult')
+        self.assertNotIn('Etag', headers)
+        expected_etag = '"%s-3"' % hashlib.md5(''.join(
+            x['hash'] for x in object_list).decode('hex')).hexdigest()
+        self.assertEqual(elem.find('ETag').text, expected_etag)
 
         self.assertEqual(self.swift.calls, [
             ('HEAD', '/v1/AUTH_test/bucket'),
             ('HEAD', '/v1/AUTH_test/bucket+segments/object/X'),
             ('GET', '/v1/AUTH_test/bucket+segments?delimiter=/&'
                     'format=json&prefix=object/X/'),
-            ('PUT', '/v1/AUTH_test/bucket/object?multipart-manifest=put'),
+            ('PUT', '/v1/AUTH_test/bucket/object?'
+                    'heartbeat=on&multipart-manifest=put'),
             ('DELETE', '/v1/AUTH_test/bucket+segments/object/X'),
         ])
 
+        _, _, headers = self.swift.calls_with_headers[-2]
+        # SLO will provide a base value
+        override_etag = '; s3_etag=%s' % expected_etag.strip('"')
+        h = 'X-Object-Sysmeta-Container-Update-Override-Etag'
+        self.assertEqual(headers.get(h), override_etag)
+
     @s3acl(s3acl_only=True)
     def test_object_multipart_upload_complete_s3acl(self):
         acl_headers = encode_acl('object', ACLPublicRead(Owner('test:tester',
@@ -1109,8 +1278,7 @@ class TestS3ApiMultiUpload(S3ApiTestCase):
         for p in elem.findall('Part'):
             partnum = int(p.find('PartNumber').text)
             self.assertEqual(p.find('LastModified').text,
-                             objects_template[partnum - 1][1][:-3]
-                             + 'Z')
+                             objects_template[partnum - 1][1][:-3] + 'Z')
             self.assertEqual(p.find('ETag').text.strip(),
                              '"%s"' % objects_template[partnum - 1][2])
             self.assertEqual(p.find('Size').text,
@@ -1199,8 +1367,7 @@ class TestS3ApiMultiUpload(S3ApiTestCase):
         for p in elem.findall('Part'):
             partnum = int(p.find('PartNumber').text)
             self.assertEqual(p.find('LastModified').text,
-                             objects_template[partnum - 1][1][:-3]
-                             + 'Z')
+                             objects_template[partnum - 1][1][:-3] + 'Z')
             self.assertEqual(p.find('ETag').text,
                              '"%s"' % objects_template[partnum - 1][2])
             self.assertEqual(p.find('Size').text,
@@ -1696,7 +1863,8 @@ class TestS3ApiMultiUpload(S3ApiTestCase):
 
     def _test_no_body(self, use_content_length=False,
                       use_transfer_encoding=False, string_to_md5=''):
-        content_md5 = md5(string_to_md5).digest().encode('base64').strip()
+        raw_md5 = hashlib.md5(string_to_md5).digest()
+        content_md5 = raw_md5.encode('base64').strip()
         with UnreadableInput(self) as fake_input:
             req = Request.blank(
                 '/bucket/object?uploadId=X',
@@ -1740,5 +1908,6 @@ class TestS3ApiMultiUploadNonUTC(TestS3ApiMultiUpload):
         os.environ['TZ'] = self.orig_tz
         time.tzset()
 
+
 if __name__ == '__main__':
     unittest.main()
