@@ -13,12 +13,9 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import hashlib
 from mock import patch, MagicMock
 import unittest
 
-from six import BytesIO
-
 from swift.common import swob
 from swift.common.swob import Request, HTTPNoContent
 from swift.common.middleware.s3api.utils import mktime
@@ -27,7 +24,7 @@ from swift.common.middleware.s3api.subresource import ACL, User, Owner, \
     Grant, encode_acl
 from test.unit.common.middleware.s3api.test_s3api import S3ApiTestCase
 from swift.common.middleware.s3api.s3request import S3Request, \
-    S3AclRequest, SigV4Request, SIGV4_X_AMZ_DATE_FORMAT, HashingInput
+    S3AclRequest, SigV4Request, SIGV4_X_AMZ_DATE_FORMAT
 from swift.common.middleware.s3api.s3response import InvalidArgument, \
     NoSuchBucket, InternalError, \
     AccessDenied, SignatureDoesNotMatch, RequestTimeTooSkewed
@@ -325,7 +322,7 @@ class TestRequest(S3ApiTestCase):
                 'swift.common.middleware.s3api.s3request.get_container_info',
                 return_value={'status': 204}) as mock_info:
             # Then all calls goes to get_container_info
-            for x in xrange(10):
+            for x in range(10):
                 info = s3_req.get_container_info(self.swift)
                 self.assertTrue('status' in info)  # sanity
                 self.assertEqual(204, info['status'])  # sanity
@@ -403,9 +400,11 @@ class TestRequest(S3ApiTestCase):
         headers = {
             'Authorization':
                 'AWS4-HMAC-SHA256 '
-                'Credential=test/20130524/US/s3/aws4_request, '
+                'Credential=test/%s/us-east-1/s3/aws4_request, '
                 'SignedHeaders=%s,'
-                'Signature=X' % ';'.join(sorted(['host', included_header])),
+                'Signature=X' % (
+                    self.get_v4_amz_date_header().split('T', 1)[0],
+                    ';'.join(sorted(['host', included_header]))),
             'X-Amz-Content-SHA256': '0123456789'}
 
         headers.update(date_header)
@@ -464,6 +463,12 @@ class TestRequest(S3ApiTestCase):
         self.assertEqual('403 Forbidden', cm.exception.message)
         self.assertIn(access_denied_message, cm.exception.body)
 
+        # near-future X-Amz-Date header
+        dt = self.get_v4_amz_date_header()
+        date_header = {'X-Amz-Date': '%d%s' % (int(dt[:4]) + 1, dt[4:])}
+        with self.assertRaises(RequestTimeTooSkewed) as cm:
+            self._test_request_timestamp_sigv4(date_header)
+
         # far-future Date header
         date_header = {'Date': 'Tue, 07 Jul 9999 21:53:04 GMT'}
         with self.assertRaises(RequestTimeTooSkewed) as cm:
@@ -553,9 +558,9 @@ class TestRequest(S3ApiTestCase):
         headers = {
             'Authorization':
                 'AWS4-HMAC-SHA256 '
-                'Credential=test/20130524/US/s3/aws4_request, '
+                'Credential=test/%s/us-east-1/s3/aws4_request, '
                 'SignedHeaders=host;x-amz-content-sha256;x-amz-date,'
-                'Signature=X',
+                'Signature=X' % self.get_v4_amz_date_header().split('T', 1)[0],
             'X-Amz-Content-SHA256': '0123456789',
             'Date': self.get_date_header(),
             'X-Amz-Date': x_amz_date}
@@ -573,9 +578,9 @@ class TestRequest(S3ApiTestCase):
         headers = {
             'Authorization':
                 'AWS4-HMAC-SHA256 '
-                'Credential=test/20130524/US/s3/aws4_request, '
+                'Credential=test/%s/us-east-1/s3/aws4_request, '
                 'SignedHeaders=host;x-amz-content-sha256,'
-                'Signature=X',
+                'Signature=X' % self.get_v4_amz_date_header().split('T', 1)[0],
             'X-Amz-Content-SHA256': '0123456789',
             'Date': self.get_date_header()}
 
@@ -592,9 +597,9 @@ class TestRequest(S3ApiTestCase):
         headers = {
             'Authorization':
                 'AWS4-HMAC-SHA256 '
-                'Credential=test/20130524/US/s3/aws4_request, '
+                'Credential=test/%s/us-east-1/s3/aws4_request, '
                 'SignedHeaders=host;x-amz-content-sha256;x-amz-date,'
-                'Signature=X',
+                'Signature=X' % self.get_v4_amz_date_header().split('T', 1)[0],
             'X-Amz-Content-SHA256': '0123456789',
             'Date': self.get_date_header()}
 
@@ -656,9 +661,9 @@ class TestRequest(S3ApiTestCase):
         headers = {
             'Authorization':
                 'AWS4-HMAC-SHA256 '
-                'Credential=test/20130524/US/s3/aws4_request, '
+                'Credential=test/%s/us-east-1/s3/aws4_request, '
                 'SignedHeaders=host;x-amz-content-sha256;x-amz-date,'
-                'Signature=X',
+                'Signature=X' % self.get_v4_amz_date_header().split('T', 1)[0],
             'X-Amz-Content-SHA256': '0123456789',
             'Date': self.get_date_header(),
             'X-Amz-Date': x_amz_date}
@@ -701,8 +706,8 @@ class TestRequest(S3ApiTestCase):
         self.assertEqual(uri, '/bucket/obj1')
         self.assertEqual(req.environ['PATH_INFO'], '/bucket/obj1')
 
-    @patch.object(S3Request, '_validate_headers', lambda *a: None)
-    def test_check_signature_sigv2(self):
+    @patch.object(S3Request, '_validate_dates', lambda *a: None)
+    def _test_check_signature_sigv2(self, secret):
         # See https://web.archive.org/web/20151226025049/http://
         # docs.aws.amazon.com//AmazonS3/latest/dev/RESTAuthentication.html
         req = Request.blank('/photos/puppy.jpg', headers={
@@ -720,8 +725,7 @@ class TestRequest(S3ApiTestCase):
             '/johnsmith/photos/puppy.jpg',
         ])
         self.assertEqual(expected_sts, sigv2_req._string_to_sign())
-        self.assertTrue(sigv2_req.check_signature(
-            'wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMPLEKEY'))
+        self.assertTrue(sigv2_req.check_signature(secret))
 
         req = Request.blank('/photos/puppy.jpg', method='PUT', headers={
             'Content-Type': 'image/jpeg',
@@ -740,8 +744,7 @@ class TestRequest(S3ApiTestCase):
             '/johnsmith/photos/puppy.jpg',
         ])
         self.assertEqual(expected_sts, sigv2_req._string_to_sign())
-        self.assertTrue(sigv2_req.check_signature(
-            'wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMPLEKEY'))
+        self.assertTrue(sigv2_req.check_signature(secret))
 
         req = Request.blank(
             '/?prefix=photos&max-keys=50&marker=puppy',
@@ -761,79 +764,46 @@ class TestRequest(S3ApiTestCase):
             '/johnsmith/',
         ])
         self.assertEqual(expected_sts, sigv2_req._string_to_sign())
-        self.assertTrue(sigv2_req.check_signature(
-            'wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMPLEKEY'))
+        self.assertTrue(sigv2_req.check_signature(secret))
 
+    def test_check_signature_sigv2(self):
+        self._test_check_signature_sigv2(
+            'wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMPLEKEY')
 
-class TestHashingInput(S3ApiTestCase):
-    def test_good(self):
-        raw = b'123456789'
-        wrapped = HashingInput(BytesIO(raw), 9, hashlib.md5,
-                               hashlib.md5(raw).hexdigest())
-        self.assertEqual(b'1234', wrapped.read(4))
-        self.assertEqual(b'56', wrapped.read(2))
-        # trying to read past the end gets us whatever's left
-        self.assertEqual(b'789', wrapped.read(4))
-        # can continue trying to read -- but it'll be empty
-        self.assertEqual(b'', wrapped.read(2))
+    def test_check_signature_sigv2_unicode_string(self):
+        self._test_check_signature_sigv2(
+            u'wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMPLEKEY')
 
-        self.assertFalse(wrapped._input.closed)
-        wrapped.close()
-        self.assertTrue(wrapped._input.closed)
+    @patch.object(S3Request, '_validate_dates', lambda *a: None)
+    def test_check_signature_multi_bytes_secret_failure(self):
+        # Test v2 check_signature with multi bytes invalid secret
+        req = Request.blank('/photos/puppy.jpg', headers={
+            'Host': 'johnsmith.s3.amazonaws.com',
+            'Date': 'Tue, 27 Mar 2007 19:36:42 +0000',
+            'Authorization': ('AWS AKIAIOSFODNN7EXAMPLE:'
+                              'bWq2s1WEIj+Ydj0vQ697zp+IXMU='),
+        })
+        sigv2_req = S3Request(req.environ, storage_domain='s3.amazonaws.com')
+        # This is a failure case with utf-8 non-ascii multi-bytes charactor
+        # but we expect to return just False instead of exceptions
+        self.assertFalse(sigv2_req.check_signature(
+            u'\u30c9\u30e9\u30b4\u30f3'))
 
-    def test_empty(self):
-        wrapped = HashingInput(BytesIO(b''), 0, hashlib.sha256,
-                               hashlib.sha256(b'').hexdigest())
-        self.assertEqual(b'', wrapped.read(4))
-        self.assertEqual(b'', wrapped.read(2))
-
-        self.assertFalse(wrapped._input.closed)
-        wrapped.close()
-        self.assertTrue(wrapped._input.closed)
-
-    def test_too_long(self):
-        raw = b'123456789'
-        wrapped = HashingInput(BytesIO(raw), 8, hashlib.md5,
-                               hashlib.md5(raw).hexdigest())
-        self.assertEqual(b'1234', wrapped.read(4))
-        self.assertEqual(b'56', wrapped.read(2))
-        # even though the hash matches, there was more data than we expected
-        with self.assertRaises(swob.Response) as raised:
-            wrapped.read(3)
-        self.assertEqual(raised.exception.status, '422 Unprocessable Entity')
-        # the error causes us to close the input
-        self.assertTrue(wrapped._input.closed)
-
-    def test_too_short(self):
-        raw = b'123456789'
-        wrapped = HashingInput(BytesIO(raw), 10, hashlib.md5,
-                               hashlib.md5(raw).hexdigest())
-        self.assertEqual(b'1234', wrapped.read(4))
-        self.assertEqual(b'56', wrapped.read(2))
-        # even though the hash matches, there was more data than we expected
-        with self.assertRaises(swob.Response) as raised:
-            wrapped.read(4)
-        self.assertEqual(raised.exception.status, '422 Unprocessable Entity')
-        self.assertTrue(wrapped._input.closed)
-
-    def test_bad_hash(self):
-        raw = b'123456789'
-        wrapped = HashingInput(BytesIO(raw), 9, hashlib.sha256,
-                               hashlib.md5(raw).hexdigest())
-        self.assertEqual(b'1234', wrapped.read(4))
-        self.assertEqual(b'5678', wrapped.read(4))
-        with self.assertRaises(swob.Response) as raised:
-            wrapped.read(4)
-        self.assertEqual(raised.exception.status, '422 Unprocessable Entity')
-        self.assertTrue(wrapped._input.closed)
-
-    def test_empty_bad_hash(self):
-        wrapped = HashingInput(BytesIO(b''), 0, hashlib.sha256, 'nope')
-        with self.assertRaises(swob.Response) as raised:
-            wrapped.read(3)
-        self.assertEqual(raised.exception.status, '422 Unprocessable Entity')
-        # the error causes us to close the input
-        self.assertTrue(wrapped._input.closed)
+        # Test v4 check_signature with multi bytes invalid secret
+        amz_date_header = self.get_v4_amz_date_header()
+        req = Request.blank('/photos/puppy.jpg', headers={
+            'Authorization':
+                'AWS4-HMAC-SHA256 '
+                'Credential=test/%s/us-east-1/s3/aws4_request, '
+                'SignedHeaders=host;x-amz-content-sha256;x-amz-date,'
+                'Signature=X' % amz_date_header.split('T', 1)[0],
+            'X-Amz-Content-SHA256': '0123456789',
+            'X-Amz-Date': amz_date_header
+        })
+        sigv4_req = SigV4Request(
+            req.environ, storage_domain='s3.amazonaws.com')
+        self.assertFalse(sigv4_req.check_signature(
+            u'\u30c9\u30e9\u30b4\u30f3'))
 
 
 if __name__ == '__main__':
